@@ -2,7 +2,7 @@
 
   dir.c -
 
-  $Author: nobu $
+  $Author: yugui $
   created at: Wed Jan  5 09:51:01 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -83,38 +83,19 @@ char *strchr(char*,char);
 # define Next(p, e, enc) (p + rb_enc_mbclen(p, e, enc))
 # define Inc(p, e, enc) ((p) = Next(p, e, enc))
 
-static int
-char_casecmp(const char *p1, const char *p2, rb_encoding *enc, const int nocase)
-{
-    const char *p1end, *p2end;
-    int c1, c2;
-
-    if (!*p1) return *p1;
-    if (!*p2) return -*p2;
-    p1end = p1 + strlen(p1);
-    p2end = p2 + strlen(p2);
-    c1 = rb_enc_codepoint(p1, p1end, enc);
-    c2 = rb_enc_codepoint(p2, p2end, enc);
-
-    if (c1 == c2) return 0;
-    if (nocase) {
-	c1 = rb_enc_toupper(c1, enc);
-	c2 = rb_enc_toupper(c2, enc);
-    }
-    return c1 - c2;
-}
-
 static char *
 bracket(
     const char *p, /* pattern (next to '[') */
+    const char *pend,
     const char *s, /* string */
+    const char *send,
     int flags,
     rb_encoding *enc)
 {
-    const char *pend = p + strlen(p);
     const int nocase = flags & FNM_CASEFOLD;
     const int escape = !(flags & FNM_NOESCAPE);
-
+    unsigned int c1, c2;
+    int r;
     int ok = 0, not = 0;
 
     if (*p == '!' || *p == '^') {
@@ -128,20 +109,42 @@ bracket(
 	    t1++;
 	if (!*t1)
 	    return NULL;
-	p = Next(t1, pend, enc);
+	p = t1 + (r = rb_enc_mbclen(t1, pend, enc));
 	if (p[0] == '-' && p[1] != ']') {
 	    const char *t2 = p + 1;
+	    int r2;
 	    if (escape && *t2 == '\\')
 		t2++;
 	    if (!*t2)
 		return NULL;
-	    p = Next(t2, pend, enc);
-	    if (!ok && char_casecmp(t1, s, enc, nocase) <= 0 && char_casecmp(s, t2, enc, nocase) <= 0)
+	    p = t2 + (r2 = rb_enc_mbclen(t2, pend, enc));
+	    if (ok) continue;
+	    if ((r <= (send-s) && memcmp(t1, s, r) == 0) ||
+		(r2 <= (send-s) && memcmp(t2, s, r) == 0)) {
 		ok = 1;
+		continue;
+	    }
+	    c1 = rb_enc_codepoint(s, send, enc);
+	    if (nocase) c1 = rb_enc_toupper(c1, enc);
+	    c2 = rb_enc_codepoint(t1, pend, enc);
+	    if (nocase) c2 = rb_enc_toupper(c2, enc);
+	    if (c1 < c2) continue;
+	    c2 = rb_enc_codepoint(t2, pend, enc);
+	    if (nocase) c2 = rb_enc_toupper(c2, enc);
+	    if (c1 > c2) continue;
 	}
-	else
-	    if (!ok && char_casecmp(t1, s, enc, nocase) == 0)
+	else {
+	    if (ok) continue;
+	    if (r <= (send-s) && memcmp(t1, s, r) == 0) {
 		ok = 1;
+		continue;
+	    }
+	    if (!nocase) continue;
+	    c1 = rb_enc_toupper(rb_enc_codepoint(s, send, enc), enc);
+	    c2 = rb_enc_toupper(rb_enc_codepoint(p, pend, enc), enc);
+	    if (c1 != c2) continue;
+	}
+	ok = 1;
     }
 
     return ok == not ? NULL : (char *)p + 1;
@@ -176,6 +179,8 @@ fnmatch_helper(
     const char *s = *scur;
     const char *send = s + strlen(s);
 
+    int r;
+
     if (period && *s == '.' && *UNESCAPE(p) != '.') /* leading period */
 	RETURN(FNM_NOMATCH);
 
@@ -204,7 +209,7 @@ fnmatch_helper(
 	    const char *t;
 	    if (ISEND(s))
 		RETURN(FNM_NOMATCH);
-	    if ((t = bracket(p + 1, s, flags, enc)) != 0) {
+	    if ((t = bracket(p + 1, pend, s, send, flags, enc)) != 0) {
 		p = t;
 		Inc(s, send, enc);
 		continue;
@@ -219,9 +224,19 @@ fnmatch_helper(
 	    RETURN(ISEND(p) ? 0 : FNM_NOMATCH);
 	if (ISEND(p))
 	    goto failed;
-	if (char_casecmp(p, s, enc, nocase) != 0)
+	r = rb_enc_precise_mbclen(p, pend, enc);
+	if (!MBCLEN_CHARFOUND_P(r))
 	    goto failed;
-	Inc(p, pend, enc);
+	if (r <= (send-s) && memcmp(p, s, r) == 0) {
+	    p += r;
+	    s += r;
+	    continue;
+	}
+	if (!nocase) goto failed;
+	if (rb_enc_toupper(rb_enc_codepoint(p, pend, enc), enc) !=
+	    rb_enc_toupper(rb_enc_codepoint(s, send, enc), enc))
+	    goto failed;
+	p += r;
 	Inc(s, send, enc);
 	continue;
 
@@ -290,17 +305,21 @@ VALUE rb_cDir;
 
 struct dir_data {
     DIR *dir;
-    char *path;
-    rb_encoding *intenc;
-    rb_encoding *extenc;
+    VALUE path;
+    rb_encoding *enc;
 };
 
+static void
+mark_dir(struct dir_data *dir)
+{
+    rb_gc_mark(dir->path);
+}
+
 static void
 free_dir(struct dir_data *dir)
 {
     if (dir) {
 	if (dir->dir) closedir(dir->dir);
-	if (dir->path) xfree(dir->path);
     }
     xfree(dir);
 }
@@ -311,12 +330,11 @@ static VALUE
 dir_s_alloc(VALUE klass)
 {
     struct dir_data *dirp;
-    VALUE obj = Data_Make_Struct(klass, struct dir_data, 0, free_dir, dirp);
+    VALUE obj = Data_Make_Struct(klass, struct dir_data, mark_dir, free_dir, dirp);
 
     dirp->dir = NULL;
-    dirp->path = NULL;
-    dirp->intenc = NULL;
-    dirp->extenc = NULL;
+    dirp->path = Qnil;
+    dirp->enc = NULL;
 
     return obj;
 }
@@ -331,68 +349,36 @@ static VALUE
 dir_initialize(int argc, VALUE *argv, VALUE dir)
 {
     struct dir_data *dp;
-    static rb_encoding *fs_encoding;
-    rb_encoding  *intencoding, *extencoding;
+    rb_encoding  *fsenc;
     VALUE dirname, opt;
-    static VALUE sym_intenc, sym_extenc;
+    static VALUE sym_enc;
 
-    if (!sym_intenc) {
-	sym_intenc = ID2SYM(rb_intern("internal_encoding"));
-	sym_extenc = ID2SYM(rb_intern("external_encoding"));
-	fs_encoding = rb_filesystem_encoding();
+    if (!sym_enc) {
+	sym_enc = ID2SYM(rb_intern("encoding"));
     }
+    fsenc = rb_filesystem_encoding();
 
-    intencoding = NULL;
-    extencoding = fs_encoding;
     rb_scan_args(argc, argv, "11", &dirname, &opt);
 
     if (!NIL_P(opt)) {
-        VALUE v, extenc=Qnil, intenc=Qnil;
+        VALUE v, enc=Qnil;
         opt = rb_convert_type(opt, T_HASH, "Hash", "to_hash");
 
-        v = rb_hash_aref(opt, sym_intenc);
-        if (!NIL_P(v)) intenc = v;
-        v = rb_hash_aref(opt, sym_extenc);
-        if (!NIL_P(v)) extenc = v;
+        v = rb_hash_aref(opt, sym_enc);
+        if (!NIL_P(v)) enc = v;
 
-	if (!NIL_P(extenc)) {
-	    extencoding = rb_to_encoding(extenc);
-	    if (!NIL_P(intenc)) {
-		intencoding = rb_to_encoding(intenc);
-		if (extencoding == intencoding) {
-		    rb_warn("Ignoring internal encoding '%s': it is identical to external encoding '%s'",
-			    RSTRING_PTR(rb_inspect(intenc)),
-			    RSTRING_PTR(rb_inspect(extenc)));
-		    intencoding = NULL;
-		}
-	    }
-	}
-	else if (!NIL_P(intenc)) {
-	    rb_raise(rb_eArgError, "External encoding must be specified when internal encoding is given");
+	if (!NIL_P(enc)) {
+	    fsenc = rb_to_encoding(enc);
 	}
     }
 
     FilePathValue(dirname);
-    {
-	rb_encoding  *dirname_encoding = rb_enc_get(dirname);
-	if (rb_usascii_encoding() != dirname_encoding
-	    && rb_ascii8bit_encoding() != dirname_encoding
-#if defined __APPLE__
-	    && rb_utf8_encoding() != dirname_encoding
-#endif
-	    && extencoding != dirname_encoding) {
-	    if (!intencoding) intencoding = dirname_encoding;
-	    dirname = rb_str_transcode(dirname, rb_enc_from_encoding(extencoding));
-	}
-    }
 
     Data_Get_Struct(dir, struct dir_data, dp);
     if (dp->dir) closedir(dp->dir);
-    if (dp->path) xfree(dp->path);
     dp->dir = NULL;
-    dp->path = NULL;
-    dp->intenc = intencoding;
-    dp->extenc = extencoding;
+    dp->path = Qnil;
+    dp->enc = fsenc;
     dp->dir = opendir(RSTRING_PTR(dirname));
     if (dp->dir == NULL) {
 	if (errno == EMFILE || errno == ENFILE) {
@@ -403,7 +389,7 @@ dir_initialize(int argc, VALUE *argv, VALUE dir)
 	    rb_sys_fail(RSTRING_PTR(dirname));
 	}
     }
-    dp->path = strdup(RSTRING_PTR(dirname));
+    dp->path = rb_str_dup_frozen(dirname);
 
     return dir;
 }
@@ -423,7 +409,7 @@ static VALUE
 dir_s_open(int argc, VALUE *argv, VALUE klass)
 {
     struct dir_data *dp;
-    VALUE dir = Data_Make_Struct(klass, struct dir_data, 0, free_dir, dp);
+    VALUE dir = Data_Make_Struct(klass, struct dir_data, mark_dir, free_dir, dp);
 
     dir_initialize(argc, argv, dir);
     if (rb_block_given_p()) {
@@ -442,8 +428,8 @@ dir_closed(void)
 static void
 dir_check(VALUE dir)
 {
-    if (!OBJ_TAINTED(dir) && rb_safe_level() >= 4)
-	rb_raise(rb_eSecurityError, "Insecure: operation on untainted Dir");
+    if (!OBJ_UNTRUSTED(dir) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: operation on trusted Dir");
     rb_check_frozen(dir);
 }
 
@@ -453,15 +439,6 @@ dir_check(VALUE dir)
     if (dirp->dir == NULL) dir_closed();\
 } while (0)
 
-static VALUE
-dir_enc_str(VALUE str, struct dir_data *dirp)
-{
-    rb_enc_associate(str, dirp->extenc);
-    if (dirp->intenc) {
-        str = rb_str_transcode(str, rb_enc_from_encoding(dirp->intenc));
-    }
-    return str;
-}
 
 /*
  *  call-seq:
@@ -475,9 +452,9 @@ dir_inspect(VALUE dir)
     struct dir_data *dirp;
 
     Data_Get_Struct(dir, struct dir_data, dirp);
-    if (dirp->path) {
+    if (!NIL_P(dirp->path)) {
 	const char *c = rb_obj_classname(dir);
-	return rb_sprintf("#<%s:%s>", c, dirp->path);
+	return rb_sprintf("#<%s:%s>", c, RSTRING_PTR(dirp->path));
     }
     return rb_funcall(dir, rb_intern("to_s"), 0, 0);
 }
@@ -497,8 +474,8 @@ dir_path(VALUE dir)
     struct dir_data *dirp;
 
     Data_Get_Struct(dir, struct dir_data, dirp);
-    if (!dirp->path) return Qnil;
-    return dir_enc_str(rb_str_new2(dirp->path), dirp);
+    if (NIL_P(dirp->path)) return Qnil;
+    return rb_str_dup(dirp->path);
 }
 
 /*
@@ -523,7 +500,7 @@ dir_read(VALUE dir)
     errno = 0;
     dp = readdir(dirp->dir);
     if (dp) {
-	return dir_enc_str(rb_tainted_str_new(dp->d_name, NAMLEN(dp)), dirp);
+	return rb_external_str_new_with_enc(dp->d_name, NAMLEN(dp), dirp->enc);
     }
     else if (errno == 0) {	/* end of stream */
 	return Qnil;
@@ -561,7 +538,7 @@ dir_each(VALUE dir)
     GetDIR(dir, dirp);
     rewinddir(dirp->dir);
     for (dp = readdir(dirp->dir); dp != NULL; dp = readdir(dirp->dir)) {
-	rb_yield(dir_enc_str(rb_tainted_str_new(dp->d_name, NAMLEN(dp)), dirp));
+	rb_yield(rb_external_str_new_with_enc(dp->d_name, NAMLEN(dp), dirp->enc));
 	if (dirp->dir == NULL) dir_closed();
     }
     return dir;
@@ -661,7 +638,7 @@ dir_rewind(VALUE dir)
 {
     struct dir_data *dirp;
 
-    if (rb_safe_level() >= 4 && !OBJ_TAINTED(dir)) {
+    if (rb_safe_level() >= 4 && !OBJ_UNTRUSTED(dir)) {
 	rb_raise(rb_eSecurityError, "Insecure: can't close");
     }
     GetDIR(dir, dirp);
@@ -825,6 +802,7 @@ dir_s_getwd(VALUE dir)
     rb_secure(4);
     path = my_getcwd();
     cwd = rb_tainted_str_new2(path);
+    rb_enc_associate(cwd, rb_filesystem_encoding());
 
     xfree(path);
     return cwd;
@@ -1151,15 +1129,16 @@ static char *
 join_path(const char *path, int dirsep, const char *name)
 {
     long len = strlen(path);
-    char *buf = GLOB_ALLOC_N(char, len+strlen(name)+(dirsep?1:0)+1);
+    long len2 = strlen(name)+(dirsep?1:0)+1;
+    char *buf = GLOB_ALLOC_N(char, len+len2);
 
     if (!buf) return 0;
     memcpy(buf, path, len);
     if (dirsep) {
-	strcpy(buf+len, "/");
-	len++;
+	buf[len++] = '/';
     }
-    strcpy(buf+len, name);
+    buf[len] = '\0';
+    strlcat(buf+len, name, len2);
     return buf;
 }
 
@@ -1340,12 +1319,13 @@ glob_helper(
 	    if (*cur) {
 		char *buf;
 		char *name;
-		name = GLOB_ALLOC_N(char, strlen((*cur)->str) + 1);
+		size_t len = strlen((*cur)->str) + 1;
+		name = GLOB_ALLOC_N(char, len);
 		if (!name) {
 		    status = -1;
 		    break;
 		}
-		strcpy(name, (*cur)->str);
+		memcpy(name, (*cur)->str, len);
 		if (escape) remove_backslashes(name, enc);
 
 		new_beg = new_end = GLOB_ALLOC_N(struct glob_pattern *, end - beg);
@@ -1465,9 +1445,7 @@ rb_glob(const char *path, void (*func)(const char *, VALUE, void *), VALUE arg)
 static void
 push_pattern(const char *path, VALUE ary, void *enc)
 {
-    VALUE vpath = rb_tainted_str_new2(path);
-    rb_enc_associate(vpath, enc);
-    rb_ary_push(ary, vpath);
+    rb_ary_push(ary, rb_external_str_new_with_enc(path, strlen(path), enc));
 }
 
 static int
@@ -1567,11 +1545,12 @@ push_glob(VALUE ary, VALUE str, int flags)
     struct glob_args args;
     rb_encoding *enc = rb_enc_get(str);
 
+    if (enc == rb_usascii_encoding()) enc = rb_filesystem_encoding();
     args.func = push_pattern;
     args.value = ary;
     args.enc = enc;
 
-    return ruby_brace_glob0(RSTRING_PTR(str), flags | GLOB_VERBOSE,
+    return ruby_brace_glob0(StringValuePtr(str), flags | GLOB_VERBOSE,
 			    rb_glob_caller, (VALUE)&args, enc);
 }
 
@@ -1611,7 +1590,7 @@ dir_globs(long argc, VALUE *argv, int flags)
     for (i = 0; i < argc; ++i) {
 	int status;
 	VALUE str = argv[i];
-	StringValue(str);
+	SafeStringValue(str);
 	status = push_glob(ary, str, flags);
 	if (status) GLOB_JUMP_TAG(status);
     }
@@ -1657,7 +1636,7 @@ dir_s_aref(int argc, VALUE *argv, VALUE obj)
  *                          match all files beginning with
  *                          <code>c</code>; <code>*c</code> will match
  *                          all files ending with <code>c</code>; and
- *                          <code>*c*</code> will match all files that
+ *                          <code>\*c\*</code> will match all files that
  *                          have <code>c</code> in them (including at
  *                          the beginning or end). Equivalent to
  *                          <code>/ .* /x</code> in regexp.
