@@ -2,8 +2,7 @@
 
   dln.c -
 
-  $Author: matz $
-  $Date: 2007-08-25 12:29:39 +0900 (Sat, 25 Aug 2007) $
+  $Author: yugui $
   created at: Tue Jan 18 17:05:06 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -43,8 +42,10 @@ void *xcalloc();
 void *xrealloc();
 #endif
 
+#define free(x) xfree(x)
+
 #include <stdio.h>
-#if defined(_WIN32) || defined(__VMS)
+#if defined(_WIN32)
 #include "missing/file.h"
 #endif
 #include <sys/types.h>
@@ -69,18 +70,6 @@ void *xrealloc();
 char *getenv();
 #endif
 
-#if defined(__VMS)
-#pragma builtins
-#include <dlfcn.h>
-#endif
-
-#ifdef __MACOS__
-# include <TextUtils.h>
-# include <CodeFragments.h>
-# include <Aliases.h>
-# include "macruby_private.h"
-#endif
-
 #if defined(__APPLE__) && defined(__MACH__)   /* Mac OS X */
 # if defined(HAVE_DLOPEN)
    /* Mac OS X with dlopen (10.3 or later) */
@@ -118,11 +107,7 @@ init_funcname_len(char **buf, const char *file)
 
     /* Load the file as an object one */
     for (slash = file-1; *file; file++) /* Find position of last '/' */
-#ifdef __MACOS__
-	if (*file == ':') slash = file;
-#else
 	if (*file == '/') slash = file;
-#endif
 
     len = strlen(FUNCNAME_PATTERN) + strlen(slash + 1);
     *buf = xmalloc(len);
@@ -347,14 +332,14 @@ sym_hash(struct exec *hdrp, struct nlist *syms)
 static int
 dln_init(const char *prog)
 {
-    char *file;
+    char *file, fbuf[MAXPATHLEN];
     int fd;
     struct exec hdr;
     struct nlist *syms;
 
     if (dln_init_p == 1) return 0;
 
-    file = dln_find_exe(prog, NULL);
+    file = dln_find_exe_r(prog, NULL, fbuf, sizeof(fbuf));
     if (file == NULL || (fd = open(file, O_RDONLY)) < 0) {
 	dln_errno = errno;
 	return -1;
@@ -910,7 +895,7 @@ char *dln_librrb_ary_path = DLN_DEFAULT_LIB_PATH;
 static int
 load_lib(const char *lib)
 {
-    char *path, *file;
+    char *path, *file, fbuf[MAXPATHLEN];
     char armagic[SARMAG];
     int fd, size;
     struct ar_hdr ahdr;
@@ -941,7 +926,7 @@ load_lib(const char *lib)
     path = getenv("DLN_LIBRARY_PATH");
     if (path == NULL) path = dln_librrb_ary_path;
 
-    file = dln_find_file(lib, path);
+    file = dln_find_file_r(lib, path, fbuf, sizeof(fbuf));
     fd = open(file, O_RDONLY);
     if (fd == -1) goto syserr;
     size = read(fd, armagic, SARMAG);
@@ -1115,15 +1100,6 @@ dln_sym(const char *name)
 #include <windows.h>
 #endif
 
-#ifdef _WIN32_WCE
-#undef FormatMessage
-#define FormatMessage FormatMessageA
-#undef LoadLibrary
-#define LoadLibrary LoadLibraryA
-#undef GetProcAddress
-#define GetProcAddress GetProcAddressA
-#endif
-
 static const char *
 dln_strerror(void)
 {
@@ -1441,7 +1417,8 @@ dln_load(const char *file)
       /* load extention module */
       img_id = load_add_on(file);
       if (img_id <= 0) {
-	rb_loaderror("Failed to load %.200s", file);
+	rb_loaderror("Failed to load add_on %.200s error_code=%x",
+	  file, img_id);
       }
       
       /* find symbol for module initialize function. */
@@ -1456,8 +1433,8 @@ dln_load(const char *file)
       if (err_stat != B_NO_ERROR) {
 	char real_name[MAXPATHLEN];
 
-	strcpy(real_name, buf);
-	strcat(real_name, "__Fv");
+	strlcpy(real_name, buf, MAXPATHLEN);
+	strlcat(real_name, "__Fv", MAXPATHLEN);
         err_stat = get_image_symbol(img_id, real_name,
 				    B_SYMBOL_TYPE_TEXT, (void **)&init_fct);
       }
@@ -1478,82 +1455,6 @@ dln_load(const char *file)
     }
 #endif /* __BEOS__*/
 
-#ifdef __MACOS__   /* Mac OS 9 or before */
-# define DLN_DEFINED
-    {
-      OSErr err;
-      FSSpec libspec;
-      CFragConnectionID connID;
-      Ptr mainAddr;
-      char errMessage[1024];
-      Boolean isfolder, didsomething;
-      Str63 fragname;
-      Ptr symAddr;
-      CFragSymbolClass class;
-      void (*init_fct)();
-      char fullpath[MAXPATHLEN];
-
-      strcpy(fullpath, file);
-
-      /* resolve any aliases to find the real file */
-      c2pstr(fullpath);
-      (void)FSMakeFSSpec(0, 0, fullpath, &libspec);
-      err = ResolveAliasFile(&libspec, 1, &isfolder, &didsomething);
-      if (err) {
-	  rb_loaderror("Unresolved Alias - %s", file);
-      }
-
-      /* Load the fragment (or return the connID if it is already loaded */
-      fragname[0] = 0;
-      err = GetDiskFragment(&libspec, 0, 0, fragname, 
-			    kLoadCFrag, &connID, &mainAddr,
-			    errMessage);
-      if (err) {
-	  p2cstr(errMessage);
-	  rb_loaderror("%s - %s",errMessage , file);
-      }
-
-      /* Locate the address of the correct init function */
-      c2pstr(buf);
-      err = FindSymbol(connID, buf, &symAddr, &class);
-      if (err) {
-	  rb_loaderror("Unresolved symbols - %s" , file);
-      }
-      init_fct = (void (*)())symAddr;
-      (*init_fct)();
-      return (void*)init_fct;
-    }
-#endif /* __MACOS__ */
-
-#if defined(__VMS)
-#define DLN_DEFINED
-    {
-	void *handle, (*init_fct)();
-	char *fname, *p1, *p2;
-
-	fname = (char *)__alloca(strlen(file)+1);
-	strcpy(fname,file);
-	if (p1 = strrchr(fname,'/'))
-	    fname = p1 + 1;
-	if (p2 = strrchr(fname,'.'))
-	    *p2 = '\0';
-
-	if ((handle = (void*)dlopen(fname, 0)) == NULL) {
-	    error = dln_strerror();
-	    goto failed;
-	}
-
-	if ((init_fct = (void (*)())dlsym(handle, buf)) == NULL) {
-	    error = DLN_ERROR();
-	    dlclose(handle);
-	    goto failed;
-	}
-	/* Call the init code */
-	(*init_fct)();
-	return handle;
-    }
-#endif /* __VMS */
-
 #ifndef DLN_DEFINED
     rb_notimplement();
 #endif
@@ -1569,71 +1470,131 @@ dln_load(const char *file)
     return 0;			/* dummy return */
 }
 
-static char *dln_find_1(const char *fname, const char *path, int exe_flag);
+static char *dln_find_1(const char *fname, const char *path, char *buf, int size, int exe_flag);
 
 char *
-dln_find_exe(const char *fname, const char *path)
+dln_find_exe_r(const char *fname, const char *path, char *buf, int size)
 {
     if (!path) {
 	path = getenv(PATH_ENV);
     }
 
     if (!path) {
-#if defined(MSDOS) || defined(_WIN32) || defined(__human68k__) || defined(__MACOS__)
+#if defined(_WIN32)
 	path = "/usr/local/bin;/usr/ucb;/usr/bin;/bin;.";
 #else
 	path = "/usr/local/bin:/usr/ucb:/usr/bin:/bin:.";
 #endif
     }
-    return dln_find_1(fname, path, 1);
+    return dln_find_1(fname, path, buf, size, 1);
+}
+
+char *
+dln_find_file_r(const char *fname, const char *path, char *buf, int size)
+{
+    if (!path) path = ".";
+    return dln_find_1(fname, path, buf, size, 0);
+}
+
+static char fbuf[MAXPATHLEN];
+
+char *
+dln_find_exe(const char *fname, const char *path)
+{
+    return dln_find_exe_r(fname, path, fbuf, sizeof(fbuf));
 }
 
 char *
 dln_find_file(const char *fname, const char *path)
 {
-#ifndef __MACOS__
-    if (!path) path = ".";
-    return dln_find_1(fname, path, 0);
-#else
-    if (!path) path = ".";
-    return _macruby_path_conv_posix_to_macos(dln_find_1(fname, path, 0));
-#endif
+    return dln_find_file_r(fname, path, fbuf, sizeof(fbuf));
 }
 
-static char fbuf[MAXPATHLEN];
-
 static char *
-dln_find_1(const char *fname, const char *path, int exe_flag /* non 0 if looking for executable. */)
+dln_find_1(const char *fname, const char *path, char *fbuf, int size,
+	   int exe_flag /* non 0 if looking for executable. */)
 {
     register const char *dp;
     register const char *ep;
     register char *bp;
     struct stat st;
-#ifdef __MACOS__
-    const char* mac_fullpath;
+    int i, fspace;
+#ifdef DOSISH
+    static const char extension[][5] = {
+	".exe", ".com", ".cmd", ".bat",
+    };
+    size_t j;
+    int is_abs = 0, has_path = 0;
+    const char *ext = 0;
+    const char *p = fname;
 #endif
 
 #define RETURN_IF(expr) if (expr) return (char *)fname;
 
     RETURN_IF(!fname);
+#ifdef DOSISH
+# ifndef CharNext
+# define CharNext(p) ((p)+1)
+# endif
+# ifdef DOSISH_DRIVE_LETTER
+    if (((p[0] | 0x20) - 'a') < 26  && p[1] == ':') {
+	p += 2;
+	is_abs = 1;
+    }
+# endif
+    switch (*p) {
+      case '/': case '\\':
+	is_abs = 1;
+	p++;
+    }
+    has_path = is_abs;
+    while (*p) {
+	switch (*p) {
+	  case '/': case '\\':
+	    has_path = 1;
+	    ext = 0;
+	    p++;
+	    break;
+	  case '.':
+	    ext = p;
+	    p++;
+	    break;
+	  default:
+	    p = CharNext(p);
+	}
+    }
+    if (ext) {
+	for (j = 0; STRCASECMP(ext, extension[j]); ) {
+	    if (++j == sizeof(extension) / sizeof(extension[0])) {
+		ext = 0;
+		break;
+	    }
+	}
+    }
+    ep = bp = 0;
+    if (!exe_flag) {
+	RETURN_IF(is_abs);
+    }
+    else if (has_path) {
+	RETURN_IF(ext);
+	i = p - fname;
+	if (i + 1 > size) goto toolong;
+	fspace = size - i - 1;
+	bp = fbuf;
+	ep = p;
+	memcpy(fbuf, fname, i + 1);
+	goto needs_extension;
+    }
+#endif
+
     RETURN_IF(fname[0] == '/');
     RETURN_IF(strncmp("./", fname, 2) == 0 || strncmp("../", fname, 3) == 0);
     RETURN_IF(exe_flag && strchr(fname, '/'));
-#ifdef DOSISH
-    RETURN_IF(fname[0] == '\\');
-# ifdef DOSISH_DRIVE_LETTER
-    RETURN_IF(strlen(fname) > 2 && fname[1] == ':');
-# endif
-    RETURN_IF(strncmp(".\\", fname, 2) == 0 || strncmp("..\\", fname, 3) == 0);
-    RETURN_IF(exe_flag && strchr(fname, '\\'));
-#endif
 
 #undef RETURN_IF
 
     for (dp = path;; dp = ++ep) {
 	register int l;
-	int i;
-	int fspace;
 
 	/* extract a component */
 	ep = strchr(dp, PATH_SEP[0]);
@@ -1643,7 +1604,7 @@ dln_find_1(const char *fname, const char *path, int exe_flag /* non 0 if looking
 	/* find the length of that component */
 	l = ep - dp;
 	bp = fbuf;
-	fspace = sizeof fbuf - 2;
+	fspace = size - 2;
 	if (l > 0) {
 	    /*
 	    **	If the length of the component is zero length,
@@ -1695,43 +1656,10 @@ dln_find_1(const char *fname, const char *path, int exe_flag /* non 0 if looking
 	}
 	memcpy(bp, fname, i + 1);
 
-#ifndef __MACOS__
-	if (stat(fbuf, &st) == 0) {
-	    if (exe_flag == 0) return fbuf;
-	    /* looking for executable */
-	    if (!S_ISDIR(st.st_mode) && eaccess(fbuf, X_OK) == 0)
-		return fbuf;
-	}
-#else
-	if (mac_fullpath = _macruby_exist_file_in_libdir_as_posix_name(fbuf)) {
-	    if (exe_flag == 0) return mac_fullpath;
-	    /* looking for executable */
-	    if (stat(mac_fullpath, &st) == 0) {
-		if (!S_ISDIR(st.st_mode) && eaccess(mac_fullpath, X_OK) == 0)
-		    return mac_fullpath;
-	    }
-	}
-#endif
 #if defined(DOSISH)
-	if (exe_flag) {
-	    static const char *const extension[] = {
-#if defined(MSDOS)
-		".com", ".exe", ".bat",
-#if defined(DJGPP)
-		".btm", ".sh", ".ksh", ".pl", ".sed",
-#endif
-#elif defined(__EMX__) || defined(_WIN32)
-		".exe", ".com", ".cmd", ".bat",
-/* end of __EMX__ or _WIN32 */
-#else
-		".r", ".R", ".x", ".X", ".bat", ".BAT",
-/* __human68k__ */
-#endif
-		(char *) NULL
-	    };
-	    int j;
-
-	    for (j = 0; extension[j]; j++) {
+	if (exe_flag && !ext) {
+	  needs_extension:
+	    for (j = 0; j < sizeof(extension) / sizeof(extension[0]); j++) {
 		if (fspace < strlen(extension[j])) {
 		    fprintf(stderr, "openpath: pathname too long (ignored)\n");
 		    fprintf(stderr, "\tDirectory \"%.*s\"\n", (int) (bp - fbuf), fbuf);
@@ -1739,18 +1667,19 @@ dln_find_1(const char *fname, const char *path, int exe_flag /* non 0 if looking
 		    continue;
 		}
 		strcpy(bp + i, extension[j]);
-#ifndef __MACOS__
 		if (stat(fbuf, &st) == 0)
 		    return fbuf;
-#else
-		if (mac_fullpath = _macruby_exist_file_in_libdir_as_posix_name(fbuf))
-		    return mac_fullpath;
-
-#endif
 	    }
+	    goto next;
 	}
-#endif /* MSDOS or _WIN32 or __human68k__ or __EMX__ */
+#endif /* _WIN32 or __EMX__ */
 
+	if (stat(fbuf, &st) == 0) {
+	    if (exe_flag == 0) return fbuf;
+	    /* looking for executable */
+	    if (!S_ISDIR(st.st_mode) && eaccess(fbuf, X_OK) == 0)
+		return fbuf;
+	}
       next:
 	/* if not, and no other alternatives, life is bleak */
 	if (*ep == '\0') {
