@@ -48,8 +48,8 @@ while TRUE
   explicit = FALSE
   n = 0
   while TRUE
-    c = getch.chr
-    if c =~ "[0-9]" then
+    c = getch
+    if c =~ /[0-9]/
       n = 10 * n + c.to_i
     else
       break
