@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_bio.c 11708 2007-02-12 23:01:19Z shyouhei $
+ * $Id: ossl_bio.c 12133 2007-04-02 19:00:23Z technorama $
  * 'OpenSSL for Ruby' team members
  * Copyright (C) 2003
  * All rights reserved.
@@ -19,16 +19,29 @@ ossl_obj2bio(VALUE obj)
     BIO *bio;
 
     if (TYPE(obj) == T_FILE) {
-	OpenFile *fptr;
+	rb_io_t *fptr;
+	FILE *fp;
+	int fd;
+
 	GetOpenFile(obj, fptr);
 	rb_io_check_readable(fptr);
-	bio = BIO_new_fp(fptr->f, BIO_NOCLOSE);
-    }       
+	if ((fd = dup(FPTR_TO_FD(fptr))) < 0){
+	    rb_sys_fail(0);
+	}
+	if (!(fp = fdopen(fd, "r"))){
+	    close(fd);
+	    rb_sys_fail(0);
+	}
+	if (!(bio = BIO_new_fp(fp, BIO_CLOSE))){
+	    fclose(fp);
+	    ossl_raise(eOSSLError, NULL);
+	}
+    }
     else {
 	StringValue(obj);
-	bio = BIO_new_mem_buf(RSTRING(obj)->ptr, RSTRING(obj)->len);
+	bio = BIO_new_mem_buf(RSTRING_PTR(obj), RSTRING_LEN(obj));
+	if (!bio) ossl_raise(eOSSLError, NULL);
     }
-    if (!bio) ossl_raise(eOSSLError, NULL);
 
     return bio;
 }
