@@ -1,5 +1,5 @@
 /*
-    $Id: strscan.c 13506 2007-09-24 08:56:24Z nobu $
+    $Id: strscan.c 25546 2009-10-28 16:11:20Z yugui $
 
     Copyright (c) 1999-2006 Minero Aoki
 
@@ -126,13 +126,21 @@ infect(VALUE str, struct strscanner *p)
     return str;
 }
 
+static VALUE
+str_new(struct strscanner *p, const char *ptr, long len)
+{
+    VALUE str = rb_str_new(ptr, len);
+    rb_enc_copy(str, p->str);
+    return str;
+}
+
 static VALUE
 extract_range(struct strscanner *p, long beg_i, long end_i)
 {
     if (beg_i > S_LEN(p)) return Qnil;
     if (end_i > S_LEN(p))
         end_i = S_LEN(p);
-    return infect(rb_str_new(S_PBEG(p) + beg_i, end_i - beg_i), p);
+    return infect(str_new(p, S_PBEG(p) + beg_i, end_i - beg_i), p);
 }
 
 static VALUE
@@ -141,7 +149,7 @@ extract_beg_len(struct strscanner *p, long beg_i, long len)
     if (beg_i > S_LEN(p)) return Qnil;
     if (beg_i + len > S_LEN(p))
         len = S_LEN(p) - beg_i;
-    return infect(rb_str_new(S_PBEG(p) + beg_i, len), p);
+    return infect(str_new(p, S_PBEG(p) + beg_i, len), p);
 }
 
 /* =======================================================================
@@ -158,7 +166,7 @@ static void
 strscan_free(struct strscanner *p)
 {
     onig_region_free(&(p->regs), 0);
-    free(p);
+    ruby_xfree(p);
 }
 
 static VALUE
@@ -313,8 +321,7 @@ strscan_set_string(VALUE self, VALUE str)
 
     Data_Get_Struct(self, struct strscanner, p);
     StringValue(str);
-    p->str = rb_str_dup(str);
-    rb_obj_freeze(p->str);
+    p->str = str;
     p->curr = 0;
     CLEAR_MATCH_STATUS(p);
     return str;
@@ -395,8 +402,11 @@ strscan_set_pos(VALUE self, VALUE v)
 static VALUE
 strscan_do_scan(VALUE self, VALUE regex, int succptr, int getstr, int headonly)
 {
+    regex_t *rb_reg_prepare_re(VALUE re, VALUE str);
     struct strscanner *p;
+    regex_t *re;
     int ret;
+    int tmpreg;
 
     Check_Type(regex, T_REGEXP);
     GET_SCANNER(self, p);
@@ -405,17 +415,31 @@ strscan_do_scan(VALUE self, VALUE regex, int succptr, int getstr, int headonly)
     if (S_RESTLEN(p) < 0) {
         return Qnil;
     }
+    re = rb_reg_prepare_re(regex, p->str);
+    tmpreg = re != RREGEXP(regex)->ptr;
+    if (!tmpreg) RREGEXP(regex)->usecnt++;
+
     if (headonly) {
-        ret = onig_match(RREGEXP(regex)->ptr, (UChar* )CURPTR(p),
+        ret = onig_match(re, (UChar* )CURPTR(p),
                          (UChar* )(CURPTR(p) + S_RESTLEN(p)),
                          (UChar* )CURPTR(p), &(p->regs), ONIG_OPTION_NONE);
     }
     else {
-        ret = onig_search(RREGEXP(regex)->ptr,
+        ret = onig_search(re,
                           (UChar* )CURPTR(p), (UChar* )(CURPTR(p) + S_RESTLEN(p)),
                           (UChar* )CURPTR(p), (UChar* )(CURPTR(p) + S_RESTLEN(p)),
                           &(p->regs), ONIG_OPTION_NONE);
     }
+    if (!tmpreg) RREGEXP(regex)->usecnt--;
+    if (tmpreg) {
+        if (RREGEXP(regex)->usecnt) {
+            onig_free(re);
+        }
+        else {
+            onig_free(RREGEXP(regex)->ptr);
+            RREGEXP(regex)->ptr = re;
+        }
+    }
 
     if (ret == -2) rb_raise(ScanError, "regexp buffer overflow");
     if (ret < 0) {
@@ -519,11 +543,11 @@ strscan_check(VALUE self, VALUE re)
 }
 
 /*
- * call-seq: scan_full(pattern, return_string_p, advance_pointer_p)
+ * call-seq: scan_full(pattern, advance_pointer_p, return_string_p)
  *
  * Tests whether the given +pattern+ is matched from the current scan pointer.
- * Returns the matched string if +return_string_p+ is true.
  * Advances the scan pointer if +advance_pointer_p+ is true.
+ * Returns the matched string if +return_string_p+ is true.
  * The match register is affected.
  *
  * "full" means "#scan with full parameters".
@@ -562,7 +586,7 @@ strscan_scan_until(VALUE self, VALUE re)
  *   s = StringScanner.new('test string')
  *   s.exist? /s/            # -> 3
  *   s.scan /test/           # -> "test"
- *   s.exist? /s/            # -> 6
+ *   s.exist? /s/            # -> 2
  *   s.exist? /e/            # -> nil
  */
 static VALUE
@@ -613,12 +637,12 @@ strscan_check_until(VALUE self, VALUE re)
 }
 
 /*
- * call-seq: search_full(pattern, return_string_p, advance_pointer_p)
+ * call-seq: search_full(pattern, advance_pointer_p, return_string_p)
  *
  * Scans the string _until_ the +pattern+ is matched.
+ * Advances the scan pointer if +advance_pointer_p+, otherwise not.
  * Returns the matched string if +return_string_p+ is true, otherwise
  * returns the number of bytes advanced.
- * Advances the scan pointer if +advance_pointer_p+, otherwise not.
  * This method does affect the match register.
  */
 static VALUE
@@ -737,7 +761,7 @@ strscan_peek(VALUE self, VALUE vlen)
 
     len = NUM2LONG(vlen);
     if (EOS_P(p))
-        return infect(rb_str_new("", 0), p);
+        return infect(str_new(p, "", 0), p);
 
     if (p->curr + len > S_LEN(p))
         len = S_LEN(p) - p->curr;
@@ -999,7 +1023,7 @@ strscan_rest(VALUE self)
 
     GET_SCANNER(self, p);
     if (EOS_P(p)) {
-        return infect(rb_str_new("", 0), p);
+        return infect(str_new(p, "", 0), p);
     }
     return extract_range(p, p->curr, S_LEN(p));
 }
@@ -1243,7 +1267,7 @@ Init_strscan()
     tmp = rb_str_new2(STRSCAN_VERSION);
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Version"), tmp);
-    tmp = rb_str_new2("$Id: strscan.c 13506 2007-09-24 08:56:24Z nobu $");
+    tmp = rb_str_new2("$Id: strscan.c 25546 2009-10-28 16:11:20Z yugui $");
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Id"), tmp);
     
