@@ -41,7 +41,13 @@ class Tk::Root<TkWindow
       }
     end
 
-    root.instance_eval(&b) if block_given?
+    if block_given?
+      if TkCore::WITH_RUBY_VM  ### Ruby 1.9 !!!!
+        root.instance_exec(root, &b)
+      else
+        root.instance_eval(&b)
+      end
+    end
     root
   end
 
