@@ -1,8 +1,7 @@
 #
 #   irb/extend-command.rb - irb extend command 
 #   	$Release Version: 0.9.5$
-#   	$Revision: 11708 $
-#   	$Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
+#   	$Revision: 26480 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -112,7 +111,7 @@ module IRB
       end
     end
 
-    # aliases = [commans_alias, flag], ...
+    # aliases = [commands_alias, flag], ...
     def self.def_extend_command(cmd_name, cmd_class, load_file = nil, *aliases)
       case cmd_class
       when Symbol
@@ -123,23 +122,32 @@ module IRB
       end
 
       if load_file
-	eval %[
+	eval <<-"EOS", binding, __FILE__, __LINE__+1
 	  def #{cmd_name}(*opts, &b)
 	    require "#{load_file}"
-	    eval %[
-	      def #{cmd_name}(*opts, &b)
-		ExtendCommand::#{cmd_class}.execute(irb_context, *opts, &b)
+	    arity = ExtendCommand::#{cmd_class}.instance_method(:execute).arity
+            args = []
+	    if arity < 0
+	      args << "*opts"
+	      arity = -arity - 1
+	    end
+	    args.unshift(*(1..arity).map {|i| "arg" + i.to_s })
+	    args << "&block"
+	    args = args.join(", ")
+	    eval <<-"EOS2", binding, __FILE__, __LINE__+1
+	      def #{cmd_name}(\#{args})
+		ExtendCommand::#{cmd_class}.execute(irb_context, \#{args})
 	      end
-	    ]
+	    EOS2
 	    send :#{cmd_name}, *opts, &b
 	  end
-	]
+	EOS
       else
-	eval %[
+	eval <<-"EOS", binding, __FILE__, __LINE__+1
 	  def #{cmd_name}(*opts, &b)
 	    ExtendCommand::#{cmd_class}.execute(irb_context, *opts, &b)
 	  end
-	]
+	EOS
       end
 
       for ali, flag in aliases
@@ -203,7 +211,7 @@ module IRB
     end
 
     def self.def_extend_command(cmd_name, load_file, *aliases)
-      Context.module_eval %[
+      Context.module_eval <<-"EOS", __FILE__, __LINE__+1
         def #{cmd_name}(*opts, &b)
 	  Context.module_eval {remove_method(:#{cmd_name})}
 	  require "#{load_file}"
@@ -212,7 +220,7 @@ module IRB
 	for ali in aliases
 	  alias_method ali, cmd_name
 	end
-      ]
+      EOS
     end
 
     CE.install_extend_commands
@@ -224,13 +232,13 @@ module IRB
       extend_method = extend_method.to_s
 
       alias_name = new_alias_name(base_method)
-      module_eval %[
+      module_eval <<-"EOS", __FILE__, __LINE__+1
         alias_method alias_name, base_method
         def #{base_method}(*opts)
 	  send :#{extend_method}, *opts
 	  send :#{alias_name}, *opts
 	end
-      ]
+      EOS
     end
 
     def def_post_proc(base_method, extend_method)
@@ -238,13 +246,13 @@ module IRB
       extend_method = extend_method.to_s
 
       alias_name = new_alias_name(base_method)
-      module_eval %[
+      module_eval <<-"EOS", __FLIE__, __LINE__+1
         alias_method alias_name, base_method
         def #{base_method}(*opts)
 	  send :#{alias_name}, *opts
 	  send :#{extend_method}, *opts
 	end
-      ]
+      EOS
     end
 
     # return #{prefix}#{name}#{postfix}<num>
