@@ -132,7 +132,7 @@
 /* stdio buffers */
 struct __sbuf {
 	unsigned char *_base;
-	int	_size;
+	size_t	_size;
 };
 
 
@@ -165,13 +165,13 @@ struct __sbuf {
 typedef	struct __sFILE {
 	unsigned char *_p;	/* current position in (some) buffer */
 #if 0
-	int	_r;		/* read space left for getc() */
+	size_t	_r;		/* read space left for getc() */
 #endif
-	int	_w;		/* write space left for putc() */
+	size_t	_w;		/* write space left for putc() */
 	short	_flags;		/* flags, below; this FILE is free if 0 */
 	short	_file;		/* fileno, if Unix descriptor, else -1 */
 	struct	__sbuf _bf;	/* the buffer (at least 1 byte, if !NULL) */
-	int	_lbfsize;	/* 0 or -_bf._size, for inline putc */
+	size_t	_lbfsize;	/* 0 or -_bf._size, for inline putc */
 	int	(*vwrite)(/* struct __sFILE*, struct __suio * */);
 } FILE;
 
@@ -238,7 +238,7 @@ static int BSD__sfvwrite(fp, uio)
 	register struct __suio *uio;
 {
 	register size_t len;
-	register char *p;
+	register const char *p;
 	register struct __siov *iov;
 	register int w;
 
@@ -357,7 +357,7 @@ BSD__sbprintf(register FILE *fp, const char *fmt, va_list ap)
  * use the given digits.
  */
 static char *
-BSD__uqtoa(register u_quad_t val, char *endp, int base, int octzero, char *xdigs)
+BSD__uqtoa(register u_quad_t val, char *endp, int base, int octzero, const char *xdigs)
 {
 	register char *cp = endp;
 	register long sval;
@@ -494,7 +494,7 @@ BSD__ultoa(register u_long val, char *endp, int base, int octzero, const char *x
 #define	BUF		(MAXEXP+MAXFRACT+1)	/* + decimal point */
 #define	DEFPREC		6
 
-static char *cvt __P((double, int, int, char *, int *, int, int *));
+static char *cvt __P((double, int, int, char *, int *, int, int *, char *));
 static int exponent __P((char *, int, int));
 
 #else /* no FLOATING_POINT */
@@ -535,10 +535,10 @@ BSD_vfprintf(FILE *fp, const char *fmt0, va_list ap)
 	char sign;		/* sign prefix (' ', '+', '-', or \0) */
 #ifdef FLOATING_POINT
 	char softsign;		/* temporary negative sign for floats */
-	double _double;		/* double precision arguments %[eEfgG] */
+	double _double = 0;	/* double precision arguments %[eEfgG] */
 	int expt;		/* integer value of exponent */
-	int expsize;		/* character count for expstr */
-	int ndig;		/* actual number of digits returned by cvt */
+	int expsize = 0;	/* character count for expstr */
+	int ndig = 0;		/* actual number of digits returned by cvt */
 	char expstr[7];		/* buffer for exponent string */
 #endif
 	u_long	ulval;		/* integer arguments %[diouxX] */
@@ -564,9 +564,9 @@ BSD_vfprintf(FILE *fp, const char *fmt0, va_list ap)
 	 * below longer.
 	 */
 #define	PADSIZE	16		/* pad chunk size */
-	static char blanks[PADSIZE] =
+	static const char blanks[PADSIZE] =
 	 {' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '};
-	static char zeroes[PADSIZE] =
+	static const char zeroes[PADSIZE] =
 	 {'0','0','0','0','0','0','0','0','0','0','0','0','0','0','0','0'};
 
 	/*
@@ -622,6 +622,7 @@ BSD_vfprintf(FILE *fp, const char *fmt0, va_list ap)
 	uio.uio_resid = 0;
 	uio.uio_iovcnt = 0;
 	ret = 0;
+	xdigs = 0;
 
 	/*
 	 * Scan the format for conversions (`%' character).
@@ -712,10 +713,16 @@ reswitch:	switch (ch) {
 		case 'h':
 			flags |= SHORTINT;
 			goto rflag;
+#if SIZEOF_PTRDIFF_T == SIZEOF_LONG
+		case 't':
+#endif
 		case 'l':
 			flags |= LONGINT;
 			goto rflag;
 #ifdef _HAVE_SANE_QUAD_
+#if SIZEOF_PTRDIFF_T == SIZEOF_LONG_LONG
+		case 't':
+#endif
 		case 'q':
 			flags |= QUADINT;
 			goto rflag;
@@ -752,6 +759,8 @@ reswitch:	switch (ch) {
 #ifdef FLOATING_POINT
 		case 'e':		/* anomalous precision */
 		case 'E':
+			if (prec != 0)
+				flags |= ALT;
 			prec = (prec == -1) ?
 				DEFPREC + 1 : prec + 1;
 			/* FALLTHROUGH */
@@ -779,9 +788,9 @@ fp_begin:		_double = va_arg(ap, double);
 			}
 			flags |= FPT;
 			cp = cvt(_double, prec, flags, &softsign,
-				&expt, ch, &ndig);
+				&expt, ch, &ndig, buf);
 			if (ch == 'g' || ch == 'G') {
-				if (expt <= -4 || expt > prec)
+				if (expt <= -4 || (expt > prec && expt > 1))
 					ch = (ch == 'g') ? 'e' : 'E';
 				else
 					ch = 'g';
@@ -797,6 +806,8 @@ fp_begin:		_double = va_arg(ap, double);
 					size = expt;
 					if (prec || flags & ALT)
 						size += prec + 1;
+				} else if (!prec) { /* "0" */
+					size = 1;
 				} else	/* "0.X" */
 					size = prec + 2;
 			} else if (expt >= ndig) {	/* fixed g fmt */
@@ -1007,13 +1018,15 @@ number:			if ((dprec = prec) >= 0)
 			if (ch >= 'f') {	/* 'f' or 'g' */
 				if (_double == 0) {
 				/* kludge for __dtoa irregularity */
-					if (prec == 0 ||
+					if (ndig <= 1 &&
 					    (flags & ALT) == 0) {
 						PRINT("0", 1);
 					} else {
 						PRINT("0.", 2);
 						PAD(ndig - 1, zeroes);
 					}
+				} else if (expt == 0 && ndig == 0 && (flags & ALT) == 0) {
+					PRINT("0", 1);
 				} else if (expt <= 0) {
 					PRINT("0.", 2);
 					PAD(-expt, zeroes);
@@ -1068,10 +1081,10 @@ error:
 extern char *BSD__dtoa __P((double, int, int, int *, int *, char **));
 
 static char *
-cvt(value, ndigits, flags, sign, decpt, ch, length)
+cvt(value, ndigits, flags, sign, decpt, ch, length, buf)
 	double value;
 	int ndigits, flags, *decpt, ch, *length;
-	char *sign;
+	char *sign, *buf;
 {
 	int mode, dsgn;
 	char *digits, *bp, *rve;
@@ -1090,6 +1103,10 @@ cvt(value, ndigits, flags, sign, decpt, ch, length)
 	    *sign = '\000';
 	}
 	digits = BSD__dtoa(value, mode, ndigits, decpt, &dsgn, &rve);
+	memcpy(buf, digits, rve - digits);
+	xfree(digits);
+	rve = buf + (rve - digits);
+	digits = buf;
 	if (flags & ALT) {	/* Print trailing zeros */
 		bp = digits + ndigits;
 		if (ch == 'f') {
