@@ -2,7 +2,7 @@
 
   numeric.c -
 
-  $Author: usa $
+  $Author: yugui $
   created at: Fri Aug 13 18:33:09 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -78,6 +78,9 @@ round(double x)
     }
     return x;
 }
+#elif defined(__BEOS__)
+/* appears to be a bug in the BeOS headers */
+double round(double x);
 #endif
 
 static ID id_coerce, id_to_i, id_eq;
@@ -199,11 +202,13 @@ rb_num_coerce_relop(VALUE x, VALUE y, ID func)
 static VALUE
 num_sadded(VALUE x, VALUE name)
 {
+    const char *nstr = rb_id2name(rb_to_id(name));
     /* ruby_frame = ruby_frame->prev; */ /* pop frame for "singleton_method_added" */
     /* Numerics should be values; singleton_methods should not be added to them */
+    rb_remove_method(rb_singleton_class(x), nstr);
     rb_raise(rb_eTypeError,
 	     "can't define singleton method \"%s\" for %s",
-	     rb_id2name(rb_to_id(name)),
+	     nstr,
 	     rb_obj_classname(x));
     return Qnil;		/* not reached */
 }
@@ -385,14 +390,14 @@ num_remainder(VALUE x, VALUE y)
 
 /*
  *  call-seq:
- *     num.scalar? -> true or false
+ *     num.real? -> true or false
  *
- *  Returns <code>true</code> if <i>num</i> is an <code>Scalar</code>
+ *  Returns <code>true</code> if <i>num</i> is a <code>Real</code>
  *  (i.e. non <code>Complex</code>).
  */
 
 static VALUE
-num_scalar_p(VALUE num)
+num_real_p(VALUE num)
 {
     return Qtrue;
 }
@@ -525,12 +530,12 @@ flo_to_s(VALUE flt)
     else if(isnan(value))
 	return rb_usascii_str_new2("NaN");
 
-    sprintf(buf, "%#.15g", value); /* ensure to print decimal point */
+    snprintf(buf, sizeof(buf), "%#.15g", value); /* ensure to print decimal point */
     if (!(e = strchr(buf, 'e'))) {
 	e = buf + strlen(buf);
     }
     if (!ISDIGIT(e[-1])) { /* reformat if ended with decimal point (ex 111111111111111.) */
-	sprintf(buf, "%#.14e", value);
+	snprintf(buf, sizeof(buf), "%#.14e", value);
 	if (!(e = strchr(buf, 'e'))) {
 	    e = buf + strlen(buf);
 	}
@@ -562,7 +567,7 @@ flo_coerce(VALUE x, VALUE y)
 static VALUE
 flo_uminus(VALUE flt)
 {
-    return DOUBLE2NUM(-RFLOAT_VALUE(flt));
+    return DBL2NUM(-RFLOAT_VALUE(flt));
 }
 
 /*
@@ -578,11 +583,11 @@ flo_plus(VALUE x, VALUE y)
 {
     switch (TYPE(y)) {
       case T_FIXNUM:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) + (double)FIX2LONG(y));
+	return DBL2NUM(RFLOAT_VALUE(x) + (double)FIX2LONG(y));
       case T_BIGNUM:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) + rb_big2dbl(y));
+	return DBL2NUM(RFLOAT_VALUE(x) + rb_big2dbl(y));
       case T_FLOAT:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) + RFLOAT_VALUE(y));
+	return DBL2NUM(RFLOAT_VALUE(x) + RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '+');
     }
@@ -601,11 +606,11 @@ flo_minus(VALUE x, VALUE y)
 {
     switch (TYPE(y)) {
       case T_FIXNUM:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) - (double)FIX2LONG(y));
+	return DBL2NUM(RFLOAT_VALUE(x) - (double)FIX2LONG(y));
       case T_BIGNUM:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) - rb_big2dbl(y));
+	return DBL2NUM(RFLOAT_VALUE(x) - rb_big2dbl(y));
       case T_FLOAT:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) - RFLOAT_VALUE(y));
+	return DBL2NUM(RFLOAT_VALUE(x) - RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '-');
     }
@@ -624,11 +629,11 @@ flo_mul(VALUE x, VALUE y)
 {
     switch (TYPE(y)) {
       case T_FIXNUM:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) * (double)FIX2LONG(y));
+	return DBL2NUM(RFLOAT_VALUE(x) * (double)FIX2LONG(y));
       case T_BIGNUM:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) * rb_big2dbl(y));
+	return DBL2NUM(RFLOAT_VALUE(x) * rb_big2dbl(y));
       case T_FLOAT:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) * RFLOAT_VALUE(y));
+	return DBL2NUM(RFLOAT_VALUE(x) * RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '*');
     }
@@ -651,12 +656,12 @@ flo_div(VALUE x, VALUE y)
     switch (TYPE(y)) {
       case T_FIXNUM:
 	f_y = FIX2LONG(y);
-	return DOUBLE2NUM(RFLOAT_VALUE(x) / (double)f_y);
+	return DBL2NUM(RFLOAT_VALUE(x) / (double)f_y);
       case T_BIGNUM:
 	d = rb_big2dbl(y);
-	return DOUBLE2NUM(RFLOAT_VALUE(x) / d);
+	return DBL2NUM(RFLOAT_VALUE(x) / d);
       case T_FLOAT:
-	return DOUBLE2NUM(RFLOAT_VALUE(x) / RFLOAT_VALUE(y));
+	return DBL2NUM(RFLOAT_VALUE(x) / RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '/');
     }
@@ -726,7 +731,7 @@ flo_mod(VALUE x, VALUE y)
 	return rb_num_coerce_bin(x, y, '%');
     }
     flodivmod(RFLOAT_VALUE(x), fy, 0, &mod);
-    return DOUBLE2NUM(mod);
+    return DBL2NUM(mod);
 }
 
 static VALUE
@@ -773,7 +778,7 @@ flo_divmod(VALUE x, VALUE y)
     }
     flodivmod(RFLOAT_VALUE(x), fy, &div, &mod);
     a = dbl2ival(div);
-    b = DOUBLE2NUM(mod);
+    b = DBL2NUM(mod);
     return rb_assoc_new(a, b);
 }
 
@@ -790,11 +795,11 @@ flo_pow(VALUE x, VALUE y)
 {
     switch (TYPE(y)) {
       case T_FIXNUM:
-	return DOUBLE2NUM(pow(RFLOAT_VALUE(x), (double)FIX2LONG(y)));
+	return DBL2NUM(pow(RFLOAT_VALUE(x), (double)FIX2LONG(y)));
       case T_BIGNUM:
-	return DOUBLE2NUM(pow(RFLOAT_VALUE(x), rb_big2dbl(y)));
+	return DBL2NUM(pow(RFLOAT_VALUE(x), rb_big2dbl(y)));
       case T_FLOAT:
-	return DOUBLE2NUM(pow(RFLOAT_VALUE(x), RFLOAT_VALUE(y)));
+	return DBL2NUM(pow(RFLOAT_VALUE(x), RFLOAT_VALUE(y)));
       default:
 	return rb_num_coerce_bin(x, y, rb_intern("**"));
     }
@@ -1130,7 +1135,7 @@ static VALUE
 flo_abs(VALUE flt)
 {
     double val = fabs(RFLOAT_VALUE(flt));
-    return DOUBLE2NUM(val);
+    return DBL2NUM(val);
 }
 
 /*
@@ -1312,7 +1317,7 @@ flo_round(int argc, VALUE *argv, VALUE num)
 	else number /= f;
     }
 
-    if (ndigits > 0) return DOUBLE2NUM(number);
+    if (ndigits > 0) return DBL2NUM(number);
 
     if (!FIXABLE(number)) {
 	return rb_dbl2big(number);
@@ -1419,6 +1424,34 @@ num_truncate(VALUE num)
 }
 
 
+int
+ruby_float_step(VALUE from, VALUE to, VALUE step, int excl)
+{
+    if (TYPE(from) == T_FLOAT || TYPE(to) == T_FLOAT || TYPE(step) == T_FLOAT) {
+	const double epsilon = DBL_EPSILON;
+	double beg = NUM2DBL(from);
+	double end = NUM2DBL(to);
+	double unit = NUM2DBL(step);
+	double n = (end - beg)/unit;
+	double err = (fabs(beg) + fabs(end) + fabs(end-beg)) / fabs(unit) * epsilon;
+	long i;
+
+	if (isinf(unit)) {
+	    if (unit > 0) rb_yield(DBL2NUM(beg));
+	}
+	else {
+	    if (err>0.5) err=0.5;
+	    n = floor(n + err);
+	    if (!excl) n++;
+	    for (i=0; i<n; i++) {
+		rb_yield(DBL2NUM(i*unit+beg));
+	    }
+	}
+	return Qtrue;
+    }
+    return Qfalse;
+}
+
 /*
  *  call-seq:
  *     num.step(limit, step ) {|i| block }     => num
@@ -1489,22 +1522,7 @@ num_step(int argc, VALUE *argv, VALUE from)
 	    }
 	}
     }
-    else if (TYPE(from) == T_FLOAT || TYPE(to) == T_FLOAT || TYPE(step) == T_FLOAT) {
-	const double epsilon = DBL_EPSILON;
-	double beg = NUM2DBL(from);
-	double end = NUM2DBL(to);
-	double unit = NUM2DBL(step);
-	double n = (end - beg)/unit;
-	double err = (fabs(beg) + fabs(end) + fabs(end-beg)) / fabs(unit) * epsilon;
-	long i;
-
-	if (err>0.5) err=0.5;
-	n = floor(n + err) + 1;
-	for (i=0; i<n; i++) {
-	    rb_yield(DOUBLE2NUM(i*unit+beg));
-	}
-    }
-    else {
+    else if (!ruby_float_step(from, to, step, Qfalse)) {
 	VALUE i = from;
 	ID cmp;
 
@@ -1543,7 +1561,7 @@ rb_num2long(VALUE val)
 	    char buf[24];
 	    char *s;
 
-	    sprintf(buf, "%-.10g", RFLOAT_VALUE(val));
+	    snprintf(buf, sizeof(buf), "%-.10g", RFLOAT_VALUE(val));
 	    if ((s = strchr(buf, ' ')) != 0) *s = '\0';
 	    rb_raise(rb_eRangeError, "float %s out of range of integer", buf);
 	}
@@ -1689,7 +1707,7 @@ rb_num2ll(VALUE val)
 	    char buf[24];
 	    char *s;
 
-	    sprintf(buf, "%-.10g", RFLOAT_VALUE(val));
+	    snprintf(buf, sizeof(buf), "%-.10g", RFLOAT_VALUE(val));
 	    if ((s = strchr(buf, ' ')) != 0) *s = '\0';
 	    rb_raise(rb_eRangeError, "float %s out of range of long long", buf);
 	}
@@ -1915,6 +1933,26 @@ int_chr(int argc, VALUE *argv, VALUE num)
     return str;
 }
 
+/*
+ *  call-seq:
+ *     int.ord    => int
+ *
+ *  Returns the int itself.
+ *
+ *     ?a.ord    #=> 97
+ *
+ *  This method is intended for compatibility to
+ *  character constant in Ruby 1.9.
+ *  For example, ?a.ord returns 97 both in 1.8 and 1.9.
+ */
+
+static VALUE
+int_ord(num)
+    VALUE num;
+{
+    return num;
+}
+
 static VALUE
 int_numerator(VALUE num)
 {
@@ -1946,66 +1984,6 @@ int_denominator(VALUE num)
  */
 
 
-/*
- * call-seq:
- *   Fixnum.induced_from(obj)    =>  fixnum
- *
- * Convert <code>obj</code> to a Fixnum. Works with numeric parameters.
- * Also works with Symbols, but this is deprecated.
- */
-
-static VALUE
-rb_fix_induced_from(VALUE klass, VALUE x)
-{
-    return rb_num2fix(x);
-}
-
-/*
- * call-seq:
- *   Integer.induced_from(obj)    =>  fixnum, bignum
- *
- * Convert <code>obj</code> to an Integer.
- */
-
-static VALUE
-rb_int_induced_from(VALUE klass, VALUE x)
-{
-    switch (TYPE(x)) {
-      case T_FIXNUM:
-      case T_BIGNUM:
-	return x;
-      case T_FLOAT:
-      case T_RATIONAL:
-	return rb_funcall(x, id_to_i, 0);
-      default:
-	rb_raise(rb_eTypeError, "failed to convert %s into Integer",
-		 rb_obj_classname(x));
-    }
-}
-
-/*
- * call-seq:
- *   Float.induced_from(obj)    =>  float
- *
- * Convert <code>obj</code> to a float.
- */
-
-static VALUE
-rb_flo_induced_from(VALUE klass, VALUE x)
-{
-    switch (TYPE(x)) {
-      case T_FIXNUM:
-      case T_BIGNUM:
-      case T_RATIONAL:
-	return rb_funcall(x, rb_intern("to_f"), 0);
-      case T_FLOAT:
-	return x;
-      default:
-	rb_raise(rb_eTypeError, "failed to convert %s into Float",
-		 rb_obj_classname(x));
-    }
-}
-
 /*
  * call-seq:
  *   -fix   =>  integer
@@ -2106,7 +2084,7 @@ fix_plus(VALUE x, VALUE y)
       case T_BIGNUM:
 	return rb_big_plus(y, x);
       case T_FLOAT:
-	return DOUBLE2NUM((double)FIX2LONG(x) + RFLOAT_VALUE(y));
+	return DBL2NUM((double)FIX2LONG(x) + RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '+');
     }
@@ -2140,7 +2118,7 @@ fix_minus(VALUE x, VALUE y)
 	x = rb_int2big(FIX2LONG(x));
 	return rb_big_minus(x, y);
       case T_FLOAT:
-	return DOUBLE2NUM((double)FIX2LONG(x) - RFLOAT_VALUE(y));
+	return DBL2NUM((double)FIX2LONG(x) - RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '-');
     }
@@ -2199,7 +2177,7 @@ fix_mul(VALUE x, VALUE y)
       case T_BIGNUM:
 	return rb_big_mul(y, x);
       case T_FLOAT:
-	return DOUBLE2NUM((double)FIX2LONG(x) * RFLOAT_VALUE(y));
+	return DBL2NUM((double)FIX2LONG(x) * RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, '*');
     }
@@ -2248,13 +2226,13 @@ static VALUE
 fix_fdiv(VALUE x, VALUE y)
 {
     if (FIXNUM_P(y)) {
-	return DOUBLE2NUM((double)FIX2LONG(x) / (double)FIX2LONG(y));
+	return DBL2NUM((double)FIX2LONG(x) / (double)FIX2LONG(y));
     }
     switch (TYPE(y)) {
       case T_BIGNUM:
-	return DOUBLE2NUM((double)FIX2LONG(x) / rb_big2dbl(y));
+	return DBL2NUM((double)FIX2LONG(x) / rb_big2dbl(y));
       case T_FLOAT:
-	return DOUBLE2NUM((double)FIX2LONG(x) / RFLOAT_VALUE(y));
+	return DBL2NUM((double)FIX2LONG(x) / RFLOAT_VALUE(y));
       default:
 	return rb_num_coerce_bin(x, y, rb_intern("fdiv"));
     }
@@ -2279,7 +2257,7 @@ fix_divide(VALUE x, VALUE y, ID op)
 
 	    if (op == '/') {
 		div = (double)FIX2LONG(x) / RFLOAT_VALUE(y);
-		return DOUBLE2NUM(div);
+		return DBL2NUM(div);
 	    }
 	    else {
 		if (RFLOAT_VALUE(y) == 0) rb_num_zerodiv();
@@ -2347,7 +2325,7 @@ fix_mod(VALUE x, VALUE y)
 	    double mod;
 
 	    flodivmod((double)FIX2LONG(x), RFLOAT_VALUE(y), 0, &mod);
-	    return DOUBLE2NUM(mod);
+	    return DBL2NUM(mod);
 	}
       default:
 	return rb_num_coerce_bin(x, y, '%');
@@ -2381,7 +2359,7 @@ fix_divmod(VALUE x, VALUE y)
 
 	    flodivmod((double)FIX2LONG(x), RFLOAT_VALUE(y), &div, &mod);
 	    a = dbl2ival(div);
-	    b = DOUBLE2NUM(mod);
+	    b = DBL2NUM(mod);
 	    return rb_assoc_new(a, b);
 	}
       default:
@@ -2447,13 +2425,13 @@ fix_pow(VALUE x, VALUE y)
 	long b = FIX2LONG(y);
 
 	if (b < 0)
-	  return rb_funcall(rb_rational_raw1(x), rb_intern("**"), 1, y);
+	    return rb_funcall(rb_rational_raw1(x), rb_intern("**"), 1, y);
 
 	if (b == 0) return INT2FIX(1);
 	if (b == 1) return x;
 	if (a == 0) {
 	    if (b > 0) return INT2FIX(0);
-	    return DOUBLE2NUM(1.0 / zero);
+	    return DBL2NUM(1.0 / zero);
 	}
 	if (a == 1) return INT2FIX(1);
 	if (a == -1) {
@@ -2468,7 +2446,7 @@ fix_pow(VALUE x, VALUE y)
       case T_BIGNUM:
 
 	if (rb_funcall(y, '<', 1, INT2FIX(0)))
-	  return rb_funcall(rb_rational_raw1(x), rb_intern("**"), 1, y);
+	    return rb_funcall(rb_rational_raw1(x), rb_intern("**"), 1, y);
 
 	if (a == 0) return INT2FIX(0);
 	if (a == 1) return INT2FIX(1);
@@ -2479,12 +2457,12 @@ fix_pow(VALUE x, VALUE y)
 	x = rb_int2big(FIX2LONG(x));
 	return rb_big_pow(x, y);
       case T_FLOAT:
-	if (RFLOAT_VALUE(y) == 0.0) return DOUBLE2NUM(1.0);
+	if (RFLOAT_VALUE(y) == 0.0) return DBL2NUM(1.0);
 	if (a == 0) {
-	    return DOUBLE2NUM(RFLOAT_VALUE(y) < 0 ? (1.0 / zero) : 0.0);
+	    return DBL2NUM(RFLOAT_VALUE(y) < 0 ? (1.0 / zero) : 0.0);
 	}
-	if (a == 1) return DOUBLE2NUM(1.0);
-	return DOUBLE2NUM(pow((double)a, RFLOAT_VALUE(y)));
+	if (a == 1) return DBL2NUM(1.0);
+	return DBL2NUM(pow((double)a, RFLOAT_VALUE(y)));
       default:
 	return rb_num_coerce_bin(x, y, rb_intern("**"));
     }
@@ -2854,7 +2832,7 @@ fix_to_f(VALUE num)
 
     val = (double)FIX2LONG(num);
 
-    return DOUBLE2NUM(val);
+    return DBL2NUM(val);
 }
 
 /*
@@ -3106,6 +3084,7 @@ void
 Init_Numeric(void)
 {
 #undef rb_intern
+#define rb_intern(str) rb_intern_const(str)
 
 #if defined(__FreeBSD__) && __FreeBSD__ < 4
     /* allow divide by zero -- Inf */
@@ -3141,9 +3120,10 @@ Init_Numeric(void)
     rb_define_method(rb_cNumeric, "modulo", num_modulo, 1);
     rb_define_method(rb_cNumeric, "remainder", num_remainder, 1);
     rb_define_method(rb_cNumeric, "abs", num_abs, 0);
+    rb_define_method(rb_cNumeric, "magnitude", num_abs, 0);
     rb_define_method(rb_cNumeric, "to_int", num_to_int, 0);
 
-    rb_define_method(rb_cNumeric, "scalar?", num_scalar_p, 0);
+    rb_define_method(rb_cNumeric, "real?", num_real_p, 0);
     rb_define_method(rb_cNumeric, "integer?", num_int_p, 0);
     rb_define_method(rb_cNumeric, "zero?", num_zero_p, 0);
     rb_define_method(rb_cNumeric, "nonzero?", num_nonzero_p, 0);
@@ -3167,11 +3147,11 @@ Init_Numeric(void)
     rb_define_method(rb_cInteger, "upto", int_upto, 1);
     rb_define_method(rb_cInteger, "downto", int_downto, 1);
     rb_define_method(rb_cInteger, "times", int_dotimes, 0);
-    rb_include_module(rb_cInteger, rb_mPrecision);
     rb_define_method(rb_cInteger, "succ", int_succ, 0);
     rb_define_method(rb_cInteger, "next", int_succ, 0);
     rb_define_method(rb_cInteger, "pred", int_pred, 0);
     rb_define_method(rb_cInteger, "chr", int_chr, -1);
+    rb_define_method(rb_cInteger, "ord", int_ord, 0);
     rb_define_method(rb_cInteger, "to_i", int_to_i, 0);
     rb_define_method(rb_cInteger, "to_int", int_to_i, 0);
     rb_define_method(rb_cInteger, "floor", int_to_i, 0);
@@ -3179,14 +3159,11 @@ Init_Numeric(void)
     rb_define_method(rb_cInteger, "truncate", int_to_i, 0);
     rb_define_method(rb_cInteger, "round", int_round, -1);
 
-    rb_cFixnum = rb_define_class("Fixnum", rb_cInteger);
-    rb_include_module(rb_cFixnum, rb_mPrecision);
-    rb_define_singleton_method(rb_cFixnum, "induced_from", rb_fix_induced_from, 1);
-    rb_define_singleton_method(rb_cInteger, "induced_from", rb_int_induced_from, 1);
-
     rb_define_method(rb_cInteger, "numerator", int_numerator, 0);
     rb_define_method(rb_cInteger, "denominator", int_denominator, 0);
 
+    rb_cFixnum = rb_define_class("Fixnum", rb_cInteger);
+
     rb_define_method(rb_cFixnum, "to_s", fix_to_s, -1);
 
     rb_define_method(rb_cFixnum, "-@", fix_uminus, 0);
@@ -3202,6 +3179,7 @@ Init_Numeric(void)
     rb_define_method(rb_cFixnum, "**", fix_pow, 1);
 
     rb_define_method(rb_cFixnum, "abs", fix_abs, 0);
+    rb_define_method(rb_cFixnum, "magnitude", fix_abs, 0);
 
     rb_define_method(rb_cFixnum, "==", fix_equal, 1);
     rb_define_method(rb_cFixnum, "<=>", fix_cmp, 1);
@@ -3231,9 +3209,6 @@ Init_Numeric(void)
     rb_undef_alloc_func(rb_cFloat);
     rb_undef_method(CLASS_OF(rb_cFloat), "new");
 
-    rb_define_singleton_method(rb_cFloat, "induced_from", rb_flo_induced_from, 1);
-    rb_include_module(rb_cFloat, rb_mPrecision);
-
     rb_define_const(rb_cFloat, "ROUNDS", INT2FIX(FLT_ROUNDS));
     rb_define_const(rb_cFloat, "RADIX", INT2FIX(FLT_RADIX));
     rb_define_const(rb_cFloat, "MANT_DIG", INT2FIX(DBL_MANT_DIG));
@@ -3242,9 +3217,9 @@ Init_Numeric(void)
     rb_define_const(rb_cFloat, "MAX_EXP", INT2FIX(DBL_MAX_EXP));
     rb_define_const(rb_cFloat, "MIN_10_EXP", INT2FIX(DBL_MIN_10_EXP));
     rb_define_const(rb_cFloat, "MAX_10_EXP", INT2FIX(DBL_MAX_10_EXP));
-    rb_define_const(rb_cFloat, "MIN", DOUBLE2NUM(DBL_MIN));
-    rb_define_const(rb_cFloat, "MAX", DOUBLE2NUM(DBL_MAX));
-    rb_define_const(rb_cFloat, "EPSILON", DOUBLE2NUM(DBL_EPSILON));
+    rb_define_const(rb_cFloat, "MIN", DBL2NUM(DBL_MIN));
+    rb_define_const(rb_cFloat, "MAX", DBL2NUM(DBL_MAX));
+    rb_define_const(rb_cFloat, "EPSILON", DBL2NUM(DBL_EPSILON));
 
     rb_define_method(rb_cFloat, "to_s", flo_to_s, 0);
     rb_define_method(rb_cFloat, "coerce", flo_coerce, 1);
@@ -3269,6 +3244,7 @@ Init_Numeric(void)
     rb_define_method(rb_cFloat, "hash", flo_hash, 0);
     rb_define_method(rb_cFloat, "to_f", flo_to_f, 0);
     rb_define_method(rb_cFloat, "abs", flo_abs, 0);
+    rb_define_method(rb_cFloat, "magnitude", flo_abs, 0);
     rb_define_method(rb_cFloat, "zero?", flo_zero_p, 0);
 
     rb_define_method(rb_cFloat, "to_i", flo_truncate, 0);
