@@ -2,7 +2,7 @@
 
   re.c -
 
-  $Author: nobu $
+  $Author: yugui $
   created at: Mon Aug  9 18:24:49 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -19,6 +19,7 @@
 VALUE rb_eRegexpError;
 
 typedef char onig_errmsg_buffer[ONIG_MAX_ERROR_MESSAGE_LEN];
+#define errcpy(err, msg) strlcpy((err), (msg), ONIG_MAX_ERROR_MESSAGE_LEN)
 
 #define BEG(no) regs->beg[no]
 #define END(no) regs->end[no]
@@ -380,8 +381,12 @@ static VALUE
 rb_reg_desc(const char *s, long len, VALUE re)
 {
     VALUE str = rb_str_buf_new2("/");
-
-    rb_enc_copy(str, re);
+    if (re && rb_enc_asciicompat(rb_enc_get(re))) {
+	rb_enc_copy(str, re);
+    }
+    else {
+	rb_enc_associate(str, rb_usascii_encoding());
+    }
     rb_reg_expr_str(str, s, len);
     rb_str_buf_cat2(str, "/");
     if (re) {
@@ -435,7 +440,9 @@ rb_reg_source(VALUE re)
 static VALUE
 rb_reg_inspect(VALUE re)
 {
-    rb_reg_check(re);
+    if (!RREGEXP(re)->ptr || !RREGEXP_SRC(re) || !RREGEXP_SRC_PTR(re)) {
+        return rb_any_to_s(re);
+    }
     return rb_reg_desc(RREGEXP_SRC_PTR(re), RREGEXP_SRC_LEN(re), re);
 }
 
@@ -1162,7 +1169,7 @@ rb_reg_preprocess(const char *p, const char *end, rb_encoding *enc,
 static void
 reg_enc_error(VALUE re, VALUE str)
 {
-    rb_raise(rb_eArgError,
+    rb_raise(rb_eEncCompatError,
 	     "incompatible encoding regexp match (%s regexp with %s string)",
 	     rb_enc_name(RREGEXP(re)->ptr->enc),
 	     rb_enc_name(rb_enc_get(str)));
@@ -1175,7 +1182,7 @@ rb_reg_prepare_enc(VALUE re, VALUE str, int warn)
 
     if (rb_enc_str_coderange(str) == ENC_CODERANGE_BROKEN) {
         rb_raise(rb_eArgError,
-            "broken %s string",
+            "invalid byte sequence in %s",
             rb_enc_name(rb_enc_get(str)));
     }
 
@@ -1262,10 +1269,10 @@ rb_reg_adjust_startpos(VALUE re, VALUE str, int pos, int reverse)
 	 string = (UChar*)RSTRING_PTR(str);
 
 	 if (range > 0) {
-	      p = onigenc_get_right_adjust_char_head(enc, string, string + pos);
+	      p = onigenc_get_right_adjust_char_head(enc, string, string + pos, string + RSTRING_LEN(str));
 	 }
 	 else {
-	      p = ONIGENC_LEFT_ADJUST_CHAR_HEAD(enc, string, string + pos);
+	      p = ONIGENC_LEFT_ADJUST_CHAR_HEAD(enc, string, string + pos, string + RSTRING_LEN(str));
 	 }
 	 return p - string;
     }
@@ -1834,13 +1841,13 @@ read_escaped_byte(const char **pp, const char *end, onig_errmsg_buffer err)
 
     retbyte = -1;
     if (p == end || *p++ != '\\') {
-        strcpy(err, "too short escaped multibyte character");
+        errcpy(err, "too short escaped multibyte character");
         return -1;
     }
 
 again:
     if (p == end) {
-        strcpy(err, "too short escape sequence");
+        errcpy(err, "too short escape sequence");
         return -1;
     }
     switch (*p++) {
@@ -1864,7 +1871,7 @@ again:
       case 'x': /* \xHH */
         code = ruby_scan_hex(p, end < p+2 ? end-p : 2, &len);
         if (len < 1) {
-            strcpy(err, "invalid hex escape");
+            errcpy(err, "invalid hex escape");
             return -1;
         }
         p += len;
@@ -1872,7 +1879,7 @@ again:
 
       case 'M': /* \M-X, \M-\C-X, \M-\cX */
         if (meta_prefix) {
-            strcpy(err, "duplicate meta escape");
+            errcpy(err, "duplicate meta escape");
             return -1;
         }
         meta_prefix = 1;
@@ -1886,17 +1893,17 @@ again:
                 break;
             }
         }
-        strcpy(err, "too short meta escape");
+        errcpy(err, "too short meta escape");
         return -1;
 
       case 'C': /* \C-X, \C-\M-X */
         if (p == end || *p++ != '-') {
-            strcpy(err, "too short control escape");
+            errcpy(err, "too short control escape");
             return -1;
         }
       case 'c': /* \cX, \c\M-X */
         if (ctrl_prefix) {
-            strcpy(err, "duplicate control escape");
+            errcpy(err, "duplicate control escape");
             return -1;
         }
         ctrl_prefix = 1;
@@ -1910,15 +1917,15 @@ again:
                 break;
             }
         }
-        strcpy(err, "too short control escape");
+        errcpy(err, "too short control escape");
         return -1;
 
       default:
-        strcpy(err, "unexpected escape sequence");
+        errcpy(err, "unexpected escape sequence");
         return -1;
     }
     if (code < 0 || 0xff < code) {
-        strcpy(err, "invalid escape code");
+        errcpy(err, "invalid escape code");
         return -1;
     }
 
@@ -1961,7 +1968,7 @@ unescape_escaped_nonascii(const char **pp, const char *end, rb_encoding *enc,
 
     l = rb_enc_precise_mbclen(chbuf, chbuf+chlen, enc);
     if (MBCLEN_INVALID_P(l)) {
-        strcpy(err, "invalid multibyte escape");
+        errcpy(err, "invalid multibyte escape");
         return -1;
     }
     if (1 < chlen || (chbuf[0] & 0x80)) {
@@ -1970,7 +1977,7 @@ unescape_escaped_nonascii(const char **pp, const char *end, rb_encoding *enc,
         if (*encp == 0)
             *encp = enc;
         else if (*encp != enc) {
-            strcpy(err, "escaped non ASCII character in UTF-8 regexp");
+            errcpy(err, "escaped non ASCII character in UTF-8 regexp");
             return -1;
         }
     }
@@ -1988,7 +1995,7 @@ check_unicode_range(unsigned long code, onig_errmsg_buffer err)
 {
     if ((0xd800 <= code && code <= 0xdfff) || /* Surrogates */
         0x10ffff < code) {
-        strcpy(err, "invalid Unicode range");
+        errcpy(err, "invalid Unicode range");
         return -1;
     }
     return 0;
@@ -2014,7 +2021,7 @@ append_utf8(unsigned long uv,
         if (*encp == 0)
             *encp = rb_utf8_encoding();
         else if (*encp != rb_utf8_encoding()) {
-            strcpy(err, "UTF-8 character in non UTF-8 regexp");
+            errcpy(err, "UTF-8 character in non UTF-8 regexp");
             return -1;
         }
     }
@@ -2037,7 +2044,7 @@ unescape_unicode_list(const char **pp, const char *end,
         if (len == 0)
             break;
         if (6 < len) { /* max 10FFFF */
-            strcpy(err, "invalid Unicode range");
+            errcpy(err, "invalid Unicode range");
             return -1;
         }
         p += len;
@@ -2049,7 +2056,7 @@ unescape_unicode_list(const char **pp, const char *end,
     }
 
     if (has_unicode == 0) {
-        strcpy(err, "invalid Unicode list");
+        errcpy(err, "invalid Unicode list");
         return -1;
     }
 
@@ -2067,12 +2074,12 @@ unescape_unicode_bmp(const char **pp, const char *end,
     unsigned long code;
 
     if (end < p+4) {
-        strcpy(err, "invalid Unicode escape");
+        errcpy(err, "invalid Unicode escape");
         return -1;
     }
     code = ruby_scan_hex(p, 4, &len);
     if (len != 4) {
-        strcpy(err, "invalid Unicode escape");
+        errcpy(err, "invalid Unicode escape");
         return -1;
     }
     if (append_utf8(code, buf, encp, err) != 0)
@@ -2092,7 +2099,7 @@ unescape_nonascii(const char *p, const char *end, rb_encoding *enc,
     while (p < end) {
         int chlen = rb_enc_precise_mbclen(p, end, enc);
         if (!MBCLEN_CHARFOUND_P(chlen)) {
-            strcpy(err, "invalid multibyte character");
+            errcpy(err, "invalid multibyte character");
             return -1;
         }
         chlen = MBCLEN_CHARFOUND_LEN(chlen);
@@ -2102,7 +2109,7 @@ unescape_nonascii(const char *p, const char *end, rb_encoding *enc,
             if (*encp == 0)
                 *encp = enc;
             else if (*encp != enc) {
-                strcpy(err, "non ASCII character in UTF-8 regexp");
+                errcpy(err, "non ASCII character in UTF-8 regexp");
                 return -1;
             }
             continue;
@@ -2111,7 +2118,7 @@ unescape_nonascii(const char *p, const char *end, rb_encoding *enc,
         switch (c = *p++) {
           case '\\':
             if (p == end) {
-                strcpy(err, "too short escape sequence");
+                errcpy(err, "too short escape sequence");
                 return -1;
             }
             switch (c = *p++) {
@@ -2141,7 +2148,7 @@ unescape_nonascii(const char *p, const char *end, rb_encoding *enc,
 
               case 'u':
                 if (p == end) {
-                    strcpy(err, "too short escape sequence");
+                    errcpy(err, "too short escape sequence");
                     return -1;
                 }
                 if (*p == '{') {
@@ -2150,7 +2157,7 @@ unescape_nonascii(const char *p, const char *end, rb_encoding *enc,
                     if (unescape_unicode_list(&p, end, buf, encp, err) != 0)
                         return -1;
                     if (p == end || *p++ != '}') {
-                        strcpy(err, "invalid Unicode list");
+                        errcpy(err, "invalid Unicode list");
                         return -1;
                     }
                     break;
@@ -2247,15 +2254,13 @@ rb_reg_preprocess_dregexp(VALUE ary)
     onig_errmsg_buffer err = "";
     int i;
     VALUE result = 0;
-    int argc = RARRAY_LEN(ary);
-    VALUE *argv = RARRAY_PTR(ary);
 
-    if (argc == 0) {
+    if (RARRAY_LEN(ary) == 0) {
         rb_raise(rb_eArgError, "no arguments given");
     }
 
-    for (i = 0; i < argc; i++) {
-        VALUE str = argv[i];
+    for (i = 0; i < RARRAY_LEN(ary); i++) {
+        VALUE str = RARRAY_PTR(ary)[i];
         VALUE buf;
         char *p, *end;
         rb_encoding *src_enc;
@@ -2299,7 +2304,7 @@ rb_reg_initialize(VALUE obj, const char *s, int len, rb_encoding *enc,
     rb_encoding *fixed_enc = 0;
     rb_encoding *a_enc = rb_ascii8bit_encoding();
 
-    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
+    if (!OBJ_UNTRUSTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify regexp");
     rb_check_frozen(obj);
     if (FL_TEST(obj, REG_LITERAL))
@@ -2308,6 +2313,11 @@ rb_reg_initialize(VALUE obj, const char *s, int len, rb_encoding *enc,
         rb_raise(rb_eTypeError, "already initialized regexp");
     re->ptr = 0;
 
+    if (rb_enc_dummy_p(enc)) {
+	    errcpy(err, "can't make regexp with dummy encoding");
+	    return -1;
+    }
+
     unescaped = rb_reg_preprocess(s, s+len, enc, &fixed_enc, err);
     if (unescaped == Qnil)
         return -1;
@@ -2315,7 +2325,7 @@ rb_reg_initialize(VALUE obj, const char *s, int len, rb_encoding *enc,
     if (fixed_enc) {
 	if ((fixed_enc != enc && (options & ARG_ENCODING_FIXED)) ||
             (fixed_enc != a_enc && (options & ARG_ENCODING_NONE))) {
-	    strcpy(err, "incompatible character encoding");
+	    errcpy(err, "incompatible character encoding");
 	    return -1;
 	}
         if (fixed_enc != a_enc) {
@@ -2353,7 +2363,7 @@ rb_reg_initialize_str(VALUE obj, VALUE str, int options, onig_errmsg_buffer err)
         rb_encoding *ascii8bit = rb_ascii8bit_encoding();
         if (enc != ascii8bit) {
             if (rb_enc_str_coderange(str) != ENC_CODERANGE_7BIT) {
-                strcpy(err, "/.../n has a non escaped non ASCII character in non ASCII-8BIT script");
+                errcpy(err, "/.../n has a non escaped non ASCII character in non ASCII-8BIT script");
                 return -1;
             }
             enc = ascii8bit;
@@ -2536,7 +2546,7 @@ reg_match_pos(VALUE re, VALUE *strp, long pos)
 		return pos;
 	    }
 	}
-	pos = rb_reg_adjust_startpos(re, str, pos, 0);
+	pos = rb_str_offset(str, pos);
     }
     return rb_reg_search(re, str, pos, 0);
 }
@@ -2565,11 +2575,13 @@ reg_match_pos(VALUE re, VALUE *strp, long pos)
  *     p rhs    #=> nil
  *
  *  This assignment is implemented in the Ruby parser.
- *  So a regexp literal is required for the assignment. 
+ *  The parser detects 'regexp-literal =~ expression' for the assignment.
+ *  The regexp must be a literal without interpolation and placed at left hand side.
+ *
  *  The assignment is not occur if the regexp is not a literal.
  *
  *     re = /(?<lhs>\w+)\s*=\s*(?<rhs>\w+)/
- *     re =~ "  x = "
+ *     re =~ "  x = y  "
  *     p lhs    # undefined local variable
  *     p rhs    # undefined local variable
  *
@@ -2580,6 +2592,11 @@ reg_match_pos(VALUE re, VALUE *strp, long pos)
  *     /(?<lhs>\w+)\s*=\s*#{rhs_pat}/ =~ "x = y"
  *     p lhs    # undefined local variable
  *
+ *  The assignment is not occur if the regexp is placed at right hand side.
+ *
+ *    "  x = y  " =~ /(?<lhs>\w+)\s*=\s*(?<rhs>\w+)/
+ *    p lhs, rhs # undefined local variable
+ *
  */
 
 VALUE
@@ -2780,10 +2797,10 @@ rb_reg_initialize_m(int argc, VALUE *argv, VALUE self)
 	    char *kcode = StringValuePtr(argv[2]);
 	    if (kcode[0] == 'n' || kcode[1] == 'N') {
 		enc = rb_ascii8bit_encoding();
-		flags |= ARG_ENCODING_FIXED;
+		flags |= ARG_ENCODING_NONE;
 	    }
 	    else {
-		rb_warning("encoding option is obsolete - %s", kcode);
+		rb_warn("encoding option is ignored - %s", kcode);
 	    }
 	}
 	str = argv[0];
@@ -2860,34 +2877,34 @@ rb_reg_quote(VALUE str)
 	  case '*': case '.': case '\\':
 	  case '?': case '+': case '^': case '$':
 	  case '#':
-	    *t++ = '\\';
+            t += rb_enc_mbcput('\\', t, enc);
 	    break;
 	  case ' ':
-	    *t++ = '\\';
-	    *t++ = ' ';
+            t += rb_enc_mbcput('\\', t, enc);
+            t += rb_enc_mbcput(' ', t, enc);
 	    continue;
 	  case '\t':
-	    *t++ = '\\';
-	    *t++ = 't';
+            t += rb_enc_mbcput('\\', t, enc);
+            t += rb_enc_mbcput('t', t, enc);
 	    continue;
 	  case '\n':
-	    *t++ = '\\';
-	    *t++ = 'n';
+            t += rb_enc_mbcput('\\', t, enc);
+            t += rb_enc_mbcput('n', t, enc);
 	    continue;
 	  case '\r':
-	    *t++ = '\\';
-	    *t++ = 'r';
+            t += rb_enc_mbcput('\\', t, enc);
+            t += rb_enc_mbcput('r', t, enc);
 	    continue;
 	  case '\f':
-	    *t++ = '\\';
-	    *t++ = 'f';
+            t += rb_enc_mbcput('\\', t, enc);
+            t += rb_enc_mbcput('f', t, enc);
 	    continue;
 	  case '\v':
-	    *t++ = '\\';
-	    *t++ = 'v';
+            t += rb_enc_mbcput('\\', t, enc);
+            t += rb_enc_mbcput('v', t, enc);
 	    continue;
 	}
-	*t++ = c;
+        t += rb_enc_mbcput(c, t, enc);
     }
     rb_str_resize(tmp, t - RSTRING_PTR(tmp));
     OBJ_INFECT(tmp, str);
@@ -3130,7 +3147,7 @@ rb_reg_regsub(VALUE str, VALUE src, struct re_registers *regs, VALUE regexp)
     rb_encoding *str_enc = rb_enc_get(str);
     rb_encoding *src_enc = rb_enc_get(src);
     int acompat = rb_enc_asciicompat(str_enc);
-#define ASCGET(s,e,cl) (acompat ? (*cl=1,s[0]) : rb_enc_ascget(s, e, cl, str_enc))
+#define ASCGET(s,e,cl) (acompat ? (*cl=1,ISASCII(s[0])?s[0]:-1) : rb_enc_ascget(s, e, cl, str_enc))
 
     p = s = RSTRING_PTR(str);
     e = s + RSTRING_LEN(str);
