@@ -451,16 +451,20 @@ class TestArray < Test::Unit::TestCase
 
   def test_clone
     for taint in [ false, true ]
-      for frozen in [ false, true ]
-        a = @cls[*(0..99).to_a]
-        a.taint  if taint
-        a.freeze if frozen
-        b = a.clone
+      for untrust in [ false, true ]
+        for frozen in [ false, true ]
+          a = @cls[*(0..99).to_a]
+          a.taint  if taint
+          a.untrust  if untrust
+          a.freeze if frozen
+          b = a.clone
 
-        assert_equal(a, b)
-        assert(a.__id__ != b.__id__)
-        assert_equal(a.frozen?, b.frozen?)
-        assert_equal(a.tainted?, b.tainted?)
+          assert_equal(a, b)
+          assert(a.__id__ != b.__id__)
+          assert_equal(a.frozen?, b.frozen?)
+          assert_equal(a.untrusted?, b.untrusted?)
+          assert_equal(a.tainted?, b.tainted?)
+        end
       end
     end
   end
@@ -475,7 +479,7 @@ class TestArray < Test::Unit::TestCase
     # Ruby 1.9 feature change:
     # Enumerable#collect without block returns an Enumerator.
     #assert_equal([1, 2, 3], @cls[1, 2, 3].collect)
-    assert_kind_of Enumerable::Enumerator, @cls[1, 2, 3].collect
+    assert_kind_of Enumerator, @cls[1, 2, 3].collect
   end
 
   # also update map!
@@ -708,6 +712,13 @@ class TestArray < Test::Unit::TestCase
                  @cls[@cls[@cls[@cls[],@cls[]],@cls[@cls[]],@cls[]],@cls[@cls[@cls[]]]].flatten)
 
     assert_raise(TypeError, "[ruby-dev:31197]") { [[]].flatten("") }
+
+    a6 = @cls[[1, 2], 3]
+    a6.taint
+    a6.untrust
+    a7 = a6.flatten
+    assert_equal(true, a7.tainted?)
+    assert_equal(true, a7.untrusted?)
   end
 
   def test_flatten!
@@ -797,6 +808,14 @@ class TestArray < Test::Unit::TestCase
     assert_equal("1,2,3", a.join(','))
 
     $, = ""
+    a = @cls[1, 2, 3]
+    a.taint
+    a.untrust
+    s = a.join
+    assert_equal(true, s.tainted?)
+    assert_equal(true, s.untrusted?)
+  ensure
+    $, = nil
   end
 
   def test_last
@@ -869,7 +888,7 @@ class TestArray < Test::Unit::TestCase
     assert_equal("aGVsbG8K\n",  @cls["hello\n"].pack("m"))
     assert_equal(",:&5L;&\\*:&5L;&\\*\n",  @cls["hello\nhello\n"].pack("u"))
 
-    assert_equal("\xc2\xa9B\xe2\x89\xa0", @cls[0xa9, 0x42, 0x2260].pack("U*"))
+    assert_equal("\u{a9 42 2260}", @cls[0xa9, 0x42, 0x2260].pack("U*"))
 
 
     format = "c2x5CCxsdils_l_a6";
@@ -1138,6 +1157,14 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[1], @cls[1].sort!)
     assert_equal(@cls[], @cls[].sort!)
+
+    a = @cls[4, 3, 2, 1]
+    a.sort! {|m, n| a.replace([9, 8, 7, 6]); m <=> n }
+    assert_equal([1, 2, 3, 4], a)
+
+    a = @cls[4, 3, 2, 1]
+    a.sort! {|m, n| a.replace([9, 8, 7]); m <=> n }
+    assert_equal([1, 2, 3, 4], a)
   end
 
   def test_sort_with_callcc
@@ -1203,8 +1230,8 @@ class TestArray < Test::Unit::TestCase
     $, = ":"
     a = @cls[1, 2, 3]
     assert_equal("[1, 2, 3]", a.to_s)
-
-    $, = ""
+  ensure
+    $, = nil
   end
 
   def test_uniq
@@ -1243,7 +1270,7 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[1,2], @cls[1, 2] | @cls[1, 2])
   end
 
- def test_combination
+  def test_combination
     assert_equal(@cls[[]], @cls[1,2,3,4].combination(0).to_a)
     assert_equal(@cls[[1],[2],[3],[4]], @cls[1,2,3,4].combination(1).to_a)
     assert_equal(@cls[[1,2],[1,3],[1,4],[2,3],[2,4],[3,4]], @cls[1,2,3,4].combination(2).to_a)
@@ -1278,6 +1305,11 @@ class TestArray < Test::Unit::TestCase
                  "edcba".each_char.to_a.permutation(5).sort)
     assert_equal(@cls[].permutation(0).to_a, @cls[[]])
 
+    a = @cls[1, 2, 3, 4]
+    b = @cls[]
+    a.permutation {|x| b << x; a.replace(@cls[9, 8, 7, 6]) }
+    assert_equal(@cls[9, 8, 7, 6], a)
+    assert_equal(@cls[1, 2, 3, 4].permutation.to_a, b)
   end
 
   def test_take
@@ -1470,6 +1502,14 @@ class TestArray < Test::Unit::TestCase
     a = []
     [1, 2, 3].zip([:a, :b], ["a", "b", "c", "d"]) {|x| a << x }
     assert_equal([[1, :a, "a"], [2, :b, "b"], [3, nil, "c"]], a)
+
+    ary = Object.new
+    def ary.to_a;   [1, 2]; end
+    assert_raise(NoMethodError){ %w(a b).zip(ary) }
+    def ary.each; [3, 4].each{|e|yield e}; end
+    assert_equal([['a', 3], ['b', 4]], %w(a b).zip(ary))
+    def ary.to_ary; [5, 6]; end
+    assert_equal([['a', 5], ['b', 6]], %w(a b).zip(ary))
   end
 
   def test_transpose
@@ -1518,9 +1558,30 @@ class TestArray < Test::Unit::TestCase
     end
   end
 
-  def test_choice
+  def test_sample
     100.times do
-      assert([0, 1, 2].include?([2, 1, 0].choice))
+      assert([0, 1, 2].include?([2, 1, 0].sample))
+      samples = [2, 1, 0].sample(2)
+      samples.each{|sample|
+        assert([0, 1, 2].include?(sample))
+      }
+    end
+
+    srand(0)
+    a = (1..18).to_a
+    (0..20).each do |n|
+      10000.times do
+        b = a.sample(n)
+        assert_equal([n, 18].min, b.uniq.size)
+        assert_equal(a, (a | b).sort)
+        assert_equal(b.sort, (a & b).sort)
+      end
+
+      h = Hash.new(0)
+      10000.times do
+        a.sample(n).each {|x| h[x] += 1 }
+      end
+      assert_operator(h.values.min * 2, :>=, h.values.max) if n != 0
     end
   end
 
@@ -1569,5 +1630,45 @@ class TestArray < Test::Unit::TestCase
 
   def test_array_subclass
     assert_equal(Array2, Array2[1,2,3].uniq.class, "[ruby-dev:34581]")
+    assert_equal(Array2, Array2[1,2][0,1].class) # embeded
+    assert_equal(Array2, Array2[*(1..100)][1..99].class) #not embeded
+  end
+
+  def test_inspect
+    a = @cls[1, 2, 3]
+    a.taint
+    a.untrust
+    s = a.inspect
+    assert_equal(true, s.tainted?)
+    assert_equal(true, s.untrusted?)
+  end
+
+  def test_initialize2
+    a = [1] * 1000
+    a.instance_eval { initialize }
+    assert_equal([], a)
+  end
+
+  def test_shift_shared_ary
+    a = (1..100).to_a
+    b = []
+    b.replace(a)
+    assert_equal((1..10).to_a, a.shift(10))
+    assert_equal((11..100).to_a, a)
+  end
+
+  def test_replace_shared_ary
+    a = [1] * 100
+    b = []
+    b.replace(a)
+    a.replace([1, 2, 3])
+    assert_equal([1, 2, 3], a)
+    assert_equal([1] * 100, b)
+  end
+
+  def test_fill_negative_length
+    a = (1..10).to_a
+    a.fill(:foo, 5, -3)
+    assert_equal((1..10).to_a, a)
   end
 end
