@@ -1,6 +1,7 @@
 require 'test/unit'
 require 'continuation'
 require 'fiber'
+require_relative 'envutil'
 
 class TestContinuation < Test::Unit::TestCase
   def test_create
@@ -50,5 +51,31 @@ class TestContinuation < Test::Unit::TestCase
       c.call
     }
   end
+
+  def test_ary_flatten
+    assert_normal_exit %q{
+      require 'continuation'
+      n = 0
+      o = Object.new
+      def o.to_ary() callcc {|k| $k = k; [1,2,3]} end
+      [10,20,o,30,o,40].flatten.inspect
+      n += 1
+      $k.call if n < 100
+    }, '[ruby-dev:34798]'
+  end
+
+  def test_marshal_dump
+    assert_normal_exit %q{
+      require 'continuation'
+      n = 0
+      o = Object.new
+      def o.marshal_dump() callcc {|k| $k = k };  "fofof" end
+      a = [1,2,3,o,4,5,6]
+      Marshal.dump(a).inspect
+      n += 1
+      $k.call if n < 100
+    }, '[ruby-dev:34802]'
+  end
+
 end
 
