@@ -120,33 +120,23 @@ class TestSuper < Test::Unit::TestCase
     def uu(a)
       class << self
         define_method(:tt) do |sym|
-          super
+          super(sym)
         end
       end
     end
   end
 
-  class B
-    def m
-      [self, "#{self.class.to_s}::m"]
-    end
-  end
-
-  class C < B
-    def self.t
-      define_method(:m) {super}
-    end
-  end
-
   def test_define_method
     a = A.new
     a.uu(12)
-    assert_equal("A#tt", a.tt(12), '[ruby-core:03856]')
-
-    bug2419 = '[ruby-core:26984]'
-    q = C.t
-    assert_raise(NoMethodError, bug2419) {q.call}
-    c = C.new
-    assert_equal([c, "#{C.to_s}::m"], c.m, bug2419)
+    assert_equal("A#tt", a.tt(12), "[ruby-core:3856]")
+    e = assert_raise(RuntimeError, "[ruby-core:24244]") {
+      lambda {
+        Class.new do
+          define_method(:a) {super}.call
+        end
+      }.call
+    }
+    assert_match(/implicit argument passing of super from method defined by define_method/, e.message)
   end
 end
