@@ -2,7 +2,7 @@
 
   variable.c -
 
-  $Author: shyouhei $
+  $Author: yugui $
   created at: Tue Apr 19 23:55:15 JST 1994
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -12,11 +12,13 @@
 **********************************************************************/
 
 #include "ruby/ruby.h"
-#include "ruby/node.h"
 #include "ruby/st.h"
 #include "ruby/util.h"
+#include "node.h"
 
 void rb_vm_change_state(void);
+void rb_vm_inc_const_missing_count(void);
+
 st_table *rb_global_tbl;
 st_table *rb_class_tbl;
 static ID autoload, classpath, tmp_classpath;
@@ -143,6 +145,7 @@ classname(VALUE klass)
     if (RCLASS_IV_TBL(klass)) {
 	if (!st_lookup(RCLASS_IV_TBL(klass), classpath, &path)) {
 	    ID classid;
+	    st_data_t n;
 
 	    CONST_ID(classid, "__classid__");
 
@@ -152,7 +155,8 @@ classname(VALUE klass)
 	    path = rb_str_dup(rb_id2str(SYM2ID(path)));
 	    OBJ_FREEZE(path);
 	    st_insert(RCLASS_IV_TBL(klass), classpath, path);
-	    st_delete(RCLASS_IV_TBL(klass), (st_data_t*)&classid, 0);
+	    n = classid;
+	    st_delete(RCLASS_IV_TBL(klass), &n, 0);
 	}
 	if (TYPE(path) != T_STRING) {
 	    rb_bug("class path is not set properly");
@@ -165,7 +169,7 @@ classname(VALUE klass)
 /*
  *  call-seq:
  *     mod.name    => string
- *  
+ *
  *  Returns the name of the module <i>mod</i>.  Returns nil for anonymous modules.
  */
 
@@ -284,9 +288,15 @@ rb_obj_classname(VALUE obj)
     return rb_class2name(CLASS_OF(obj));
 }
 
+#define global_variable rb_global_variable
+
+#define gvar_getter_t rb_gvar_getter_t
+#define gvar_setter_t rb_gvar_setter_t
+#define gvar_marker_t rb_gvar_marker_t
+
 struct trace_var {
     int removed;
-    void (*func)();
+    void (*func)(VALUE arg, VALUE val);
     VALUE data;
     struct trace_var *next;
 };
@@ -294,9 +304,9 @@ struct trace_var {
 struct global_variable {
     int   counter;
     void *data;
-    VALUE (*getter)();
-    void  (*setter)();
-    void  (*marker)();
+    gvar_getter_t *getter;
+    gvar_setter_t *setter;
+    gvar_marker_t *marker;
     int block_trace;
     struct trace_var *trace;
 };
@@ -306,17 +316,19 @@ struct global_entry {
     ID id;
 };
 
-static VALUE undef_getter(ID id);
-static void  undef_setter(VALUE val, ID id, void *data, struct global_variable *var);
-static void  undef_marker(void);
+#define undef_getter	rb_gvar_undef_getter
+#define undef_setter	rb_gvar_undef_setter
+#define undef_marker	rb_gvar_undef_marker
 
-static VALUE val_getter(ID id, VALUE val);
-static void  val_setter(VALUE val, ID id, void *data, struct global_variable *var);
-static void  val_marker(VALUE data);
+#define val_getter	rb_gvar_val_getter
+#define val_setter	rb_gvar_val_setter
+#define val_marker	rb_gvar_val_marker
 
-static VALUE var_getter(ID id, VALUE *var);
-static void  var_setter(VALUE val, ID id, VALUE *var);
-static void  var_marker(VALUE *var);
+#define var_getter	rb_gvar_var_getter
+#define var_setter	rb_gvar_var_setter
+#define var_marker	rb_gvar_var_marker
+
+#define readonly_setter rb_gvar_readonly_setter
 
 struct global_entry*
 rb_global_entry(ID id)
@@ -346,15 +358,15 @@ rb_global_entry(ID id)
     return entry;
 }
 
-static VALUE
-undef_getter(ID id)
+VALUE
+undef_getter(ID id, void *data, struct global_variable *var)
 {
     rb_warning("global variable `%s' not initialized", rb_id2name(id));
 
     return Qnil;
 }
 
-static void
+void
 undef_setter(VALUE val, ID id, void *data, struct global_variable *var)
 {
     var->getter = val_getter;
@@ -364,50 +376,52 @@ undef_setter(VALUE val, ID id, void *data, struct global_variable *var)
     var->data = (void*)val;
 }
 
-static void
-undef_marker(void)
+void
+undef_marker(VALUE *var)
 {
 }
 
-static VALUE
-val_getter(ID id, VALUE val)
+VALUE
+val_getter(ID id, void *data, struct global_variable *var)
 {
-    return val;
+    return (VALUE)data;
 }
 
-static void
+void
 val_setter(VALUE val, ID id, void *data, struct global_variable *var)
 {
     var->data = (void*)val;
 }
 
-static void
-val_marker(VALUE data)
+void
+val_marker(VALUE *var)
 {
+    VALUE data = (VALUE)var;
     if (data) rb_gc_mark_maybe(data);
 }
 
-static VALUE
-var_getter(ID id, VALUE *var)
+VALUE
+var_getter(ID id, void *data, struct global_variable *gvar)
 {
+    VALUE *var = data;
     if (!var) return Qnil;
     return *var;
 }
 
-static void
-var_setter(VALUE val, ID id, VALUE *var)
+void
+var_setter(VALUE val, ID id, void *data, struct global_variable *gvar)
 {
-    *var = val;
+    *(VALUE *)data = val;
 }
 
-static void
+void
 var_marker(VALUE *var)
 {
     if (var) rb_gc_mark_maybe(*var);
 }
 
-static void
-readonly_setter(VALUE val, ID id, void *var)
+void
+readonly_setter(VALUE val, ID id, void *data, struct global_variable *gvar)
 {
     rb_name_error(id, "%s is a read-only variable", rb_id2name(id));
 }
@@ -456,18 +470,13 @@ rb_define_hooked_variable(
     VALUE (*getter)(ANYARGS),
     void  (*setter)(ANYARGS))
 {
-    struct global_variable *gvar;
-    ID id;
-    VALUE tmp;
-    
-    if (var)
-        tmp = *var;
+    volatile VALUE tmp = var ? *var : Qnil;
+    ID id = global_id(name);
+    struct global_variable *gvar = rb_global_entry(id)->var;
 
-    id = global_id(name);
-    gvar = rb_global_entry(id)->var;
     gvar->data = (void*)var;
-    gvar->getter = getter?getter:var_getter;
-    gvar->setter = setter?setter:var_setter;
+    gvar->getter = getter?(gvar_getter_t *)getter:var_getter;
+    gvar->setter = setter?(gvar_setter_t *)setter:var_setter;
     gvar->marker = var_marker;
 
     RB_GC_GUARD(tmp);
@@ -506,7 +515,7 @@ rb_trace_eval(VALUE cmd, VALUE val)
  *  call-seq:
  *     trace_var(symbol, cmd )             => nil
  *     trace_var(symbol) {|val| block }    => nil
- *  
+ *
  *  Controls tracing of assignments to global variables. The parameter
  *  +symbol_ identifies the variable (as either a string name or a
  *  symbol identifier). _cmd_ (which may be a string or a
@@ -514,13 +523,13 @@ rb_trace_eval(VALUE cmd, VALUE val)
  *  is assigned. The block or +Proc+ object receives the
  *  variable's new value as a parameter. Also see
  *  <code>Kernel::untrace_var</code>.
- *     
+ *
  *     trace_var :$_, proc {|v| puts "$_ is now '#{v}'" }
  *     $_ = "hello"
  *     $_ = ' there'
- *     
+ *
  *  <em>produces:</em>
- *     
+ *
  *     $_ is now 'hello'
  *     $_ is now ' there'
  */
@@ -578,7 +587,7 @@ remove_trace(struct global_variable *var)
 /*
  *  call-seq:
  *     untrace_var(symbol [, cmd] )   => array or nil
- *  
+ *
  *  Removes tracing for the specified command on the given global
  *  variable and returns +nil+. If no command is specified,
  *  removes all tracing for that variable and returns an array
@@ -714,9 +723,9 @@ gvar_i(ID key, struct global_entry *entry, VALUE ary)
 /*
  *  call-seq:
  *     global_variables    => array
- *  
+ *
  *  Returns an array of the names of global variables.
- *     
+ *
  *     global_variables.grep /std/   #=> [:$stdin, :$stdout, :$stderr]
  */
 
@@ -952,7 +961,7 @@ ivar_get(VALUE obj, ID id, int warn)
         len = ROBJECT_NUMIV(obj);
         ptr = ROBJECT_IVPTR(obj);
         iv_index_tbl = ROBJECT_IV_INDEX_TBL(obj);
-        if (!iv_index_tbl) break; 
+        if (!iv_index_tbl) break;
         if (!st_lookup(iv_index_tbl, id, &index)) break;
         if (len <= index) break;
         val = ptr[index];
@@ -995,7 +1004,7 @@ rb_ivar_set(VALUE obj, ID id, VALUE val)
     long i, len;
     int ivar_extended;
 
-    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
+    if (!OBJ_UNTRUSTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
     if (OBJ_FROZEN(obj)) rb_error_frozen("object");
     switch (TYPE(obj)) {
@@ -1162,11 +1171,11 @@ ivar_i(ID key, VALUE val, VALUE ary)
 /*
  *  call-seq:
  *     obj.instance_variables    => array
- *  
+ *
  *  Returns an array of instance variable names for the receiver. Note
  *  that simply defining an accessor does not create the corresponding
  *  instance variable.
- *     
+ *
  *     class Fred
  *       attr_accessor :a1
  *       def initialize
@@ -1189,10 +1198,10 @@ rb_obj_instance_variables(VALUE obj)
 /*
  *  call-seq:
  *     obj.remove_instance_variable(symbol)    => obj
- *  
+ *
  *  Removes the named instance variable from <i>obj</i>, returning that
  *  variable's value.
- *     
+ *
  *     class Dummy
  *       attr_reader :var
  *       def initialize
@@ -1212,11 +1221,12 @@ VALUE
 rb_obj_remove_instance_variable(VALUE obj, VALUE name)
 {
     VALUE val = Qnil;
-    ID id = rb_to_id(name);
+    const ID id = rb_to_id(name);
+    st_data_t n, v;
     struct st_table *iv_index_tbl;
     st_data_t index;
 
-    if (!OBJ_TAINTED(obj) && rb_safe_level() >= 4)
+    if (!OBJ_UNTRUSTED(obj) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify instance variable");
     if (OBJ_FROZEN(obj)) rb_error_frozen("object");
     if (!rb_is_instance_id(id)) {
@@ -1237,8 +1247,9 @@ rb_obj_remove_instance_variable(VALUE obj, VALUE name)
 	break;
       case T_CLASS:
       case T_MODULE:
-	if (RCLASS_IV_TBL(obj) && st_delete(RCLASS_IV_TBL(obj), (st_data_t*)&id, &val)) {
-	    return val;
+	n = id;
+	if (RCLASS_IV_TBL(obj) && st_delete(RCLASS_IV_TBL(obj), &n, &v)) {
+	    return (VALUE)v;
 	}
 	break;
       default:
@@ -1286,7 +1297,7 @@ const_missing(VALUE klass, ID id)
  *  assumed to be in file <code>fred.rb</code>). If found, it returns the
  *  value of the loaded class. It therefore implements a perverse
  *  kind of autoload facility.
- *  
+ *
  *    def Object.const_missing(name)
  *      @looked_for ||= {}
  *      str_name = name.to_s
@@ -1298,7 +1309,7 @@ const_missing(VALUE klass, ID id)
  *      return klass if klass
  *      raise "Class not found: #{name}"
  *    end
- *  
+ *
  */
 
 VALUE
@@ -1356,83 +1367,93 @@ rb_autoload(VALUE mod, ID id, const char *file)
 static NODE*
 autoload_delete(VALUE mod, ID id)
 {
-    VALUE val;
-    st_data_t load = 0;
+    st_data_t val, load = 0, n = id;
 
-    st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, 0);
+    st_delete(RCLASS_IV_TBL(mod), &n, 0);
     if (st_lookup(RCLASS_IV_TBL(mod), autoload, &val)) {
-	struct st_table *tbl = check_autoload_table(val);
+	struct st_table *tbl = check_autoload_table((VALUE)val);
 
-	st_delete(tbl, (st_data_t*)&id, &load);
+	st_delete(tbl, &n, &load);
 
 	if (tbl->num_entries == 0) {
-	    DATA_PTR(val) = 0;
-	    st_free_table(tbl);
-	    id = autoload;
-	    if (st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
-		rb_gc_force_recycle(val);
-	    }
+	    n = autoload;
+	    st_delete(RCLASS_IV_TBL(mod), &n, &val);
 	}
     }
 
     return (NODE *)load;
 }
 
-VALUE
-rb_autoload_load(VALUE klass, ID id)
+static VALUE
+autoload_provided(VALUE arg)
 {
-    VALUE file;
-    NODE *load = autoload_delete(klass, id);
-
-    if (!load || !(file = load->nd_lit)) {
-	return Qfalse;
-    }
-    return rb_require_safe(file, load->nd_nth);
+    const char **p = (const char **)arg;
+    return rb_feature_provided(*p, p);
 }
 
 static VALUE
-autoload_file(VALUE mod, ID id)
+reset_safe(VALUE safe)
 {
-    VALUE val, file;
+    rb_set_safe_level_force((int)safe);
+    return safe;
+}
+
+static NODE *
+autoload_node(VALUE mod, ID id, int noload)
+{
+    VALUE file;
     struct st_table *tbl;
-    st_data_t load;
+    st_data_t val;
+    NODE *load;
+    const char *loading;
+    int safe;
 
     if (!st_lookup(RCLASS_IV_TBL(mod), autoload, &val) ||
-	!(tbl = check_autoload_table(val)) || !st_lookup(tbl, id, &load)) {
-	return Qnil;
+	!(tbl = check_autoload_table((VALUE)val)) || !st_lookup(tbl, (st_data_t)id, &val)) {
+	return 0;
     }
-    file = ((NODE *)load)->nd_lit;
+    load = (NODE *)val;
+    file = load->nd_lit;
     Check_Type(file, T_STRING);
     if (!RSTRING_PTR(file) || !*RSTRING_PTR(file)) {
 	rb_raise(rb_eArgError, "empty file name");
     }
-    if (!rb_provided(RSTRING_PTR(file))) {
-	return file;
+    loading = RSTRING_PTR(file);
+    safe = rb_safe_level();
+    rb_set_safe_level_force(0);
+    if (!rb_ensure(autoload_provided, (VALUE)&loading, reset_safe, (VALUE)safe)) {
+	return load;
     }
+    if (!noload && loading) {
+	return load;
+    }
+    return 0;
+}
 
-    /* already loaded but not defined */
-    st_delete(tbl, (st_data_t*)&id, 0);
-    if (!tbl->num_entries) {
-	DATA_PTR(val) = 0;
-	st_free_table(tbl);
-	id = autoload;
-	if (st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
-	    rb_gc_force_recycle(val);
-	}
-    }
-    return Qnil;
+VALUE
+rb_autoload_load(VALUE klass, ID id)
+{
+    VALUE file;
+    NODE *load = autoload_node(klass, id, 0);
+
+    if (!load) return Qfalse;
+    file = load->nd_lit;
+    return rb_require_safe(file, load->nd_nth);
 }
 
 VALUE
 rb_autoload_p(VALUE mod, ID id)
 {
     struct st_table *tbl = RCLASS_IV_TBL(mod);
-    VALUE val;
+    st_data_t val;
+    NODE *load;
+    VALUE file;
 
     if (!tbl || !st_lookup(tbl, id, &val) || val != Qundef) {
 	return Qnil;
     }
-    return autoload_file(mod, id);
+    load = autoload_node(mod, id, 0);
+    return load && (file = load->nd_lit) ? file : Qnil;
 }
 
 static VALUE
@@ -1444,9 +1465,12 @@ rb_const_get_0(VALUE klass, ID id, int exclude, int recurse)
     tmp = klass;
   retry:
     while (RTEST(tmp)) {
+	VALUE am = 0;
 	while (RCLASS_IV_TBL(tmp) && st_lookup(RCLASS_IV_TBL(tmp),id,&value)) {
 	    if (value == Qundef) {
-		if (!RTEST(rb_autoload_load(tmp, id))) break;
+		if (am == tmp) break;
+		am = tmp;
+		rb_autoload_load(tmp, id);
 		continue;
 	    }
 	    if (exclude && tmp == rb_cObject && klass != rb_cObject) {
@@ -1464,7 +1488,9 @@ rb_const_get_0(VALUE klass, ID id, int exclude, int recurse)
 	goto retry;
     }
 
-    return const_missing(klass, id);
+    value = const_missing(klass, id);
+    rb_vm_inc_const_missing_count();
+    return value;
 }
 
 VALUE
@@ -1488,7 +1514,7 @@ rb_const_get_at(VALUE klass, ID id)
 /*
  *  call-seq:
  *     remove_const(sym)   => obj
- *  
+ *
  *  Removes the definition of the given constant, returning that
  *  constant's value. Predefined classes and singleton objects (such as
  *  <i>true</i>) cannot be removed.
@@ -1497,19 +1523,21 @@ rb_const_get_at(VALUE klass, ID id)
 VALUE
 rb_mod_remove_const(VALUE mod, VALUE name)
 {
-    ID id = rb_to_id(name);
+    const ID id = rb_to_id(name);
     VALUE val;
+    st_data_t v, n = id;
 
     rb_vm_change_state();
 
     if (!rb_is_const_id(id)) {
 	rb_name_error(id, "`%s' is not allowed as a constant name", rb_id2name(id));
     }
-    if (!OBJ_TAINTED(mod) && rb_safe_level() >= 4)
+    if (!OBJ_UNTRUSTED(mod) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't remove constant");
     if (OBJ_FROZEN(mod)) rb_error_frozen("class/module");
 
-    if (RCLASS_IV_TBL(mod) && st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
+    if (RCLASS_IV_TBL(mod) && st_delete(RCLASS_IV_TBL(mod), &n, &v)) {
+	val = (VALUE)v;
 	if (val == Qundef) {
 	    autoload_delete(mod, id);
 	    val = Qnil;
@@ -1586,7 +1614,7 @@ rb_const_list(void *data)
 /*
  *  call-seq:
  *     mod.constants(inherit=true)    => array
- *  
+ *
  *  Returns an array of the names of the constants accessible in
  *  <i>mod</i>. This includes the names of constants in any included
  *  modules (example at start of section), unless the <i>all</i>
@@ -1629,7 +1657,7 @@ rb_const_defined_0(VALUE klass, ID id, int exclude, int recurse)
   retry:
     while (tmp) {
 	if (RCLASS_IV_TBL(tmp) && st_lookup(RCLASS_IV_TBL(tmp), id, &value)) {
-	    if (value == Qundef && NIL_P(autoload_file(klass, id)))
+	    if (value == Qundef && !autoload_node(klass, id, 1))
 		return Qfalse;
 	    return Qtrue;
 	}
@@ -1667,8 +1695,8 @@ mod_av_set(VALUE klass, ID id, VALUE val, int isconst)
 {
     const char *dest = isconst ? "constant" : "class variable";
 
-    if (!OBJ_TAINTED(klass) && rb_safe_level() >= 4)
-      rb_raise(rb_eSecurityError, "Insecure: can't set %s", dest);
+    if (!OBJ_UNTRUSTED(klass) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't set %s", dest);
     if (OBJ_FROZEN(klass)) {
 	if (BUILTIN_TYPE(klass) == T_MODULE) {
 	    rb_error_frozen("module");
@@ -1691,7 +1719,7 @@ mod_av_set(VALUE klass, ID id, VALUE val, int isconst)
 	}
     }
 
-    if(isconst){
+    if (isconst){
 	rb_vm_change_state();
     }
     st_insert(RCLASS_IV_TBL(klass), id, val);
@@ -1869,9 +1897,9 @@ cv_i(ID key, VALUE value, VALUE ary)
 /*
  *  call-seq:
  *     mod.class_variables   => array
- *  
+ *
  *  Returns an array of the names of class variables in <i>mod</i>.
- *     
+ *
  *     class One
  *       @@var1 = 1
  *     end
@@ -1896,19 +1924,19 @@ rb_mod_class_variables(VALUE obj)
 /*
  *  call-seq:
  *     remove_class_variable(sym)    => obj
- *  
+ *
  *  Removes the definition of the <i>sym</i>, returning that
  *  constant's value.
- *     
+ *
  *     class Dummy
  *       @@var = 99
  *       puts @@var
  *       remove_class_variable(:@@var)
- *       puts(defined? @@var)
+ *       p(defined? @@var)
  *     end
- *     
+ *
  *  <em>produces:</em>
- *     
+ *
  *     99
  *     nil
  */
@@ -1916,18 +1944,18 @@ rb_mod_class_variables(VALUE obj)
 VALUE
 rb_mod_remove_cvar(VALUE mod, VALUE name)
 {
-    ID id = rb_to_id(name);
-    VALUE val;
+    const ID id = rb_to_id(name);
+    st_data_t val, n = id;
 
     if (!rb_is_class_id(id)) {
 	rb_name_error(id, "wrong class variable name %s", rb_id2name(id));
     }
-    if (!OBJ_TAINTED(mod) && rb_safe_level() >= 4)
+    if (!OBJ_UNTRUSTED(mod) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't remove class variable");
     if (OBJ_FROZEN(mod)) rb_error_frozen("class/module");
 
-    if (RCLASS_IV_TBL(mod) && st_delete(RCLASS_IV_TBL(mod), (st_data_t*)&id, &val)) {
-	return val;
+    if (RCLASS_IV_TBL(mod) && st_delete(RCLASS_IV_TBL(mod), &n, &val)) {
+	return (VALUE)val;
     }
     if (rb_cvar_defined(mod, id)) {
 	rb_name_error(id, "cannot remove %s for %s",
