@@ -1,15 +1,13 @@
-#define RUBY_VERSION "1.9.0"
-#define RUBY_RELEASE_DATE "2008-03-01"
-#define RUBY_VERSION_CODE 190
-#define RUBY_RELEASE_CODE 20080301
+#define RUBY_VERSION "1.9.1"
+#define RUBY_RELEASE_DATE "2009-01-30"
 #define RUBY_PATCHLEVEL 0
 
 #define RUBY_VERSION_MAJOR 1
 #define RUBY_VERSION_MINOR 9
-#define RUBY_VERSION_TEENY 0
-#define RUBY_RELEASE_YEAR 2008
-#define RUBY_RELEASE_MONTH 3
-#define RUBY_RELEASE_DAY 1
+#define RUBY_VERSION_TEENY 1
+#define RUBY_RELEASE_YEAR 2009
+#define RUBY_RELEASE_MONTH 1
+#define RUBY_RELEASE_DAY 30
 
 #ifdef RUBY_EXTERN
 RUBY_EXTERN const char ruby_version[];
@@ -25,30 +23,30 @@ RUBY_EXTERN const char ruby_copyright[];
 #define RUBY_BIRTH_MONTH 2
 #define RUBY_BIRTH_DAY 24
 
+#define RUBY_PATCHLEVEL_STR "p"STRINGIZE(RUBY_PATCHLEVEL)
+
 #ifndef RUBY_REVISION
-#include "revision.h"
+# include "revision.h"
 #endif
-#ifndef RUBY_REVISION
-#define RUBY_REVISION 0
+# ifndef RUBY_REVISION
+# define RUBY_REVISION 0
 #endif
 
-#if RUBY_VERSION_TEENY > 0 && RUBY_PATCHLEVEL < 5000 && !RUBY_REVISION
-#define RUBY_RELEASE_STR "patchlevel"
-#define RUBY_RELEASE_NUM RUBY_PATCHLEVEL
+#if RUBY_REVISION
+# ifdef RUBY_BRANCH_NAME
+#  define RUBY_REVISION_STR " "RUBY_BRANCH_NAME" "STRINGIZE(RUBY_REVISION)
+# else
+#  define RUBY_REVISION_STR " revision "STRINGIZE(RUBY_REVISION)
+# endif
 #else
-#ifdef RUBY_BRANCH_NAME
-#define RUBY_RELEASE_STR RUBY_BRANCH_NAME
-#else
-#define RUBY_RELEASE_STR "revision"
-#endif
-#define RUBY_RELEASE_NUM RUBY_REVISION
+# define RUBY_REVISION_STR ""
 #endif
 
 # define RUBY_DESCRIPTION	    \
     "ruby "RUBY_VERSION		    \
-    " ("RUBY_RELEASE_DATE" "	    \
-    RUBY_RELEASE_STR" "		    \
-    STRINGIZE(RUBY_RELEASE_NUM)") " \
+    RUBY_PATCHLEVEL_STR             \
+    " ("RUBY_RELEASE_DATE	    \
+    RUBY_REVISION_STR") "	    \
     "["RUBY_PLATFORM"]"
 # define RUBY_COPYRIGHT 	    \
     "ruby - Copyright (C) "	    \
